using System;
using Fissum.Core.Configuration;
using System.ComponentModel;

namespace Fissum.TrayApp.Lib
{
    public class RibbonConfiguration : INotifyPropertyChanged
    {
        protected GlobalConfiguration Config { get; private set; }

        /// <summary>
        /// Initializes a new instance of the WindowConfiguration class.
        /// </summary>
        /// <param name="config"></param>
        /// <param name="windowName"></param>
        /// <param name="defaultWidth"></param>
        /// <param name="defaultHeight"></param>
        public RibbonConfiguration(GlobalConfiguration config)
        {
            if (config == null)
                throw new ArgumentNullException("config", "config is null.");
            Config = config;
        }

        public bool IsMinimized
        {
            get
            {
                return Config.GetParameterValue("ribbon", "IsMinimized", true);
            }
            set
            {
                Config.SetParameterValue("ribbon", "IsMinimized", value);
                OnPropertyChanged("IsMinimized");
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        #region OnPropertyChanged
        /// <summary>
        /// Triggers the PropertyChanged event.
        /// </summary>
        public virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
        #endregion
    }
}