using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.UI.Core;
using System.Windows.Input;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using Fissum.TrayApp.Lib;
using System.Text.RegularExpressions;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Framework.Common;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Fissum.Core.Configuration;
using Fissum.Core.Repository;

namespace Fissum.TrayApp.ViewModel
{
    internal class TPCViewModel : AsyncViewModelBase
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(TPCViewModel));
        
        private TPCConfiguration _tpcConfig;
        private TfsTeamProjectCollection _tpc;

        public ICommand Connect { get; private set; }

        public bool IsOnline { get; private set; }

        public ICommand ConnectAsyncCommand { get; private set; }

        private void InitializeViewModel(TPCConfiguration tpcConfig)
        {
            if (tpcConfig == null)
                throw new ArgumentNullException("tpcConfig", "tpcConfig is null.");

            _tpcConfig = tpcConfig;
            _tpc = TPCRepository.Current[tpcConfig.Uri];

            ConnectAsyncCommand = new AsyncRelayCommand<object, object>(this,
                (p, c) => DoConnect(), p => true, (r, p) => OnConnectCommandFinished(r.Completed));
        }
        /// <summary>
        /// Initializes a new instance of the TPCViewModel class.
        /// </summary>
        /// <param name="tpc"></param>        
        public TPCViewModel(TPCConfiguration tpcConfig)
            : base()
        {
            InitializeViewModel(tpcConfig);
        }

        private void DoConnect()
        {
            try
            {
                _tpc.Authenticate();
            }
            catch (Exception ex)
            {
                _log.Error("Error during authentication action", ex);
            }
        }

        private void OnConnectCommandFinished(bool connectCommandFinished)
        {
            IsOnline = _tpc.HasAuthenticated;
            RaisePropertyChangedSafe(() => IsOnline);
        }

    }

#if false
    internal class ConfigEditionViewModel : CoreViewModelBase
    {

        public ConfigEditionViewModel(GlobalConfiguration configuration)
            : base()
        {

        }



    }

    internal class SelectableViewModel : AsyncViewModelBase
    {
        private bool _selected = false;
        protected SelectableViewModel():base()
        {

        }

        public bool Selected
        {
            get
            {
                return _selected;
            }
            set
            {
                if (_selected == value)
                    return;
                _selected = value;
                RaisePropertyChangedSafe(() => Selected);
            }
        }




    }
#endif

#if false

    internal class TCPConfigEditionViewModel : CoreViewModelBase
    {
        private TfsTeamProjectCollection _tpc;

        public List<ProjectConfigEditionViewModel> ProjectList { get; private set; }


  
        /// <summary>
        /// Initializes a new instance of the TCPConfigEditionViewModel class.
        /// </summary>
        /// <param name="ConfigEditionViewModel"></param>
        public TCPConfigEditionViewModel(TfsTeamProjectCollection tpc, ConfigEditionViewModel parent)
            : base(parent)
        {
            _tpc = tpc;
            FillProjectList();
            ProjectList = new List<ProjectConfigEditionViewModel>();
        }




    }


    internal class ProjectConfigEditionViewModel : SelectableViewModel
    {
        Project _project;



        private void FillQueryHierarchy()
        {
            
        }
        /// <summary>
        /// Initializes a new instance of the ProjectConfigEditionViewModel class.
        /// </summary>
        /// <param name="project"></param>
        public ProjectConfigEditionViewModel(Project project, TCPConfigEditionViewModel parent)
            : base(parent)
        {
            _project = project;
            FillQueryHierarchy();
        }
    }

    internal class QueryFolderEditionViewModel : CoreViewModelBase
    {        
        ProjectConfigEditionViewModel _project;
        List<QueryFolderEditionViewModel> _subFolderList;
        List<QueryConfigEditionViewModel> _queryList;

        public QueryFolderEditionViewModel(ProjectConfigEditionViewModel project)
            : base(project)
        {
            _subFolderList = new List<QueryFolderEditionViewModel>();
            _queryList = new List<QueryConfigEditionViewModel>();
        }


    }

    internal class QueryConfigEditionViewModel : SelectableViewModel
    {
        public QueryConfigEditionViewModel ParentQuery { get; private set; }
        public ProjectConfigEditionViewModel Project { get; private set; }

        /// <summary>
        /// Initializes a new instance of the QueryConfigEditionViewModel class.
        /// </summary>
        public QueryConfigEditionViewModel(QueryConfigEditionViewModel parentQuery, ProjectConfigEditionViewModel project):
            base(project)
        {
            ParentQuery = parentQuery;
            this.Project = project;
        }
     
 
         

    }
#endif
}
