using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using Fissum.UI.Core;
using System.Windows.Threading;

namespace Fissum.UI.Core.Menu
{
    public class MenuItemBase : AsyncViewModelBase, IMenuItem
    {

        private void InitializeMenu()
        {
            IsCheckable = false;
            IsChecked = false;
            Visible = true;
            IsSeparator = false;
        }

        public MenuItemBase()
        {
            InitializeMenu();
        }

        #region IMenuItem Members

        public IEnumerable<IMenuItem> Items
        {
            get;
            set;
        }

        private object _icon;
        public object Icon
        {
            get
            {
                return _icon;
            }
            set
            {
                if (_icon == value)
                    return;

                _icon = value;
                RaisePropertyChangedSafe(() => Icon);
            }
        }

        private bool _isCheckable;
        public bool IsCheckable
        {
            get
            {
                return _isCheckable;
            }
            set
            {
                if (_isCheckable == value)
                    return;
                _isCheckable = value;
                RaisePropertyChangedSafe(() => IsCheckable);
            }
        }

        private bool _isChecked;
        public bool IsChecked
        {
            get
            {
                return _isChecked;
            }
            set
            {
                if (_isChecked == value)
                    return;

                _isChecked = value;
                RaisePropertyChangedSafe(() => IsChecked);
            }
        }

        private bool _isEnabled;
        public bool IsEnabled
        {
            get
            {
                return _isEnabled;
            }
            set
            {
                if (_isEnabled == value)
                    return;
                _isEnabled = value;
                RaisePropertyChangedSafe(() => IsEnabled);
            }
        }

        private bool _visible;
        public virtual bool Visible
        {
            get
            {
                return _visible;
            }
            set
            {
                if (value == _visible)
                    return;

                _visible = value;
                RaisePropertyChangedSafe(() => Visible);
            }
        }

        public bool IsSeparator
        {
            get;
            set;
        }

        public object ToolTip
        {
            get;
            set;
        }

        public ICommand Command
        {
            get;
            set;
        }

        public object CommandParameter
        {
            get;
            set;
        }

        #endregion
    }
}