using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Windows.Threading;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Diagnostics;

namespace Fissum.UI.Core
{
    public class AsyncRelayCommand<PARAMETER, CONTEXT> : ICommand where CONTEXT : new()
    {
        readonly AsyncViewModelBase _asyncViewModel;
        readonly Action<PARAMETER, CONTEXT> _execute;
        readonly Predicate<PARAMETER> _canExecute;
        readonly Action<ExecutionResult, CONTEXT> _onExecuted;

        /// <summary>
        /// Initializes a new instance of the AsyncDelegateCommand class.
        /// </summary>
        /// <param name="asyncViewModel"></param>
        /// <param name="execute"></param>
        /// <param name="onExecuted"></param>
        public AsyncRelayCommand(AsyncViewModelBase asyncViewModel,
                                    Action<PARAMETER, CONTEXT> execute,
                                    Predicate<PARAMETER> canExecute,
                                    Action<ExecutionResult, CONTEXT> onExecuted)
        {
            if (asyncViewModel == null)
                throw new ArgumentNullException("asyncViewModel", "asyncViewModel is null.");
            if (execute == null)
                throw new ArgumentNullException("execute", "execute is null.");
            if (canExecute == null)
                throw new ArgumentNullException("canExecute", "canExecute is null.");
            if (onExecuted == null)
                throw new ArgumentNullException("onExecuted", "onExecuted is null.");

            _execute = execute;
            _asyncViewModel = asyncViewModel;
            _canExecute = canExecute;
            _onExecuted = onExecuted;
        }

        public void ExecuteWithParameter(PARAMETER typedParameter)
        {

            _asyncViewModel.ExecuteAsync(_execute, _onExecuted, typedParameter);
        }

        #region ICommand Members

        [DebuggerStepThrough]
        public bool CanExecute(object parameter)
        {
            return !_asyncViewModel.IsLoading && _canExecute((PARAMETER)parameter);
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }



        public void Execute(object parameter)
        {
            PARAMETER typedParameter = (PARAMETER)parameter;
            ExecuteWithParameter(typedParameter);
        }

        #endregion


    }
}
