﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Windows;
using Hardcodet.Wpf.TaskbarNotification;
using GalaSoft.MvvmLight.Messaging;
using Fissum.TrayApp.Model;
using Fissum.TrayApp.ViewModel;
using Fissum.TrayApp.Lib;
using GalaSoft.MvvmLight.Threading;
using Fissum.UI.Core;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using Fissum.TrayApp.Shell;
using System.Diagnostics;
using Fissum.TrayApp.Windows;
using Fissum.Core;
using Fissum.Core.Repository;
using Microsoft.Practices.Unity;
using Fissum.TrayApp.Messages;
using Fissum.TrayApp.ViewModel.Debug;

namespace Fissum.TrayApp
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {

        #region Init

        public void InitializeApplication()
        {

            InitializeConfiguration();
            InitializeModels();
            InitializeMedia();
            InitializeMessageRegistration();
            MainWindow = new RootShell();
            MainWindow.DataContext = new MainViewModel();
        }

        private void InitializeMedia()
        {

            Favorite16Image = new BitmapImage(new Uri("pack://application:,,,/Media/fav16.png"));
            NotFavorite16Image = new BitmapImage(new Uri("pack://application:,,,/Media/notfav16.png"));
            TFS16Image = new BitmapImage(new Uri("pack://application:,,,/Media/tfs16.png"));
            TFS16DisabledImage = new BitmapImage(new Uri("pack://application:,,,/Media/tfs_disabled16.png"));
            Refresh16Image = new BitmapImage(new Uri("pack://application:,,,/Media/Refresh16.png"));
            Refresh32Image = new BitmapImage(new Uri("pack://application:,,,/Media/Refresh32.png"));
            QueryTypeList16Image = new BitmapImage(new Uri("pack://application:,,,/Media/QueryType_List16.png"));
            QueryTypeOneHop16Image = new BitmapImage(new Uri("pack://application:,,,/Media/QueryType_OneHop16.png"));
            QueryTypeTree16Image = new BitmapImage(new Uri("pack://application:,,,/Media/QueryType_Tree16.png"));
            New16Image = new BitmapImage(new Uri("pack://application:,,,/Media/New16.png"));
            Folder16Image = new BitmapImage(new Uri("pack://application:,,,/Media/folder16.png"));
            Info16Image = new BitmapImage(new Uri("pack://application:,,,/Media/Info16.png"));
            Bug16Image = new BitmapImage(new Uri("pack://application:,,,/Media/bug16.png"));
        }


        private void InitializeConfiguration()
        {
            var configurationFile = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "fissum.config");
            ConfigurationRepository.IntializeRepository(configurationFile);
        }


        private void SaveConfiguration()
        {
            ConfigurationRepository.Current.SaveConfiguration();
        }

        private void InitializeModels()
        {

            _windows = new List<IShellWindow>();
            TPCModel.InitializeFromConfig();
        }

        #endregion

        #region Message handling

        private void InitializeMessageRegistration()
        {
            var messenger = UnityRepository.Container.Resolve<Messenger>();

            messenger.Register<Messages.OpenLinkInBrowserMessage>(this, message => HandleOpenLinkInBrowseMessage(message.Uri));
            messenger.Register<Messages.UserSimpleRequestMessage>(this, message => HandleSimpleRequestMessage(message));
            messenger.Register<Messages.ShowAboutWindowMessage>(this, message => HandleShowAboutWindowMessage(message));
            messenger.Register<Messages.ShowQueryMessage>(this, message => HandleShowQueryMessage(message));
            messenger.Register<Messages.ShowWorkItemMessage>(this, message => HandleShowWorkItemMessage(message));
            messenger.Register<Messages.ShowDebugWindowMessage>(this, message => HandleShowDebugMessage(message));
        }

        private void HandleShowWorkItemMessage(ShowWorkItemMessage message)
        {
            if (message.WorkItem.IsNew)
            {
                var window = new RibbonShell();
                window.ViewModel = new WorkItemViewModel(message.WorkItem);
                window.Show();

                _windows.Add(window);
                return;
            }

            Tuple<Window, WorkItemViewModel> tuple = GetShellWindows<WorkItemViewModel>().FirstOrDefault(t => t.Item2.WorkItemModel.Id == message.WorkItem.Id);

            if (tuple == null)
            {
                var window = new RibbonShell();
                window.DataContext = new WorkItemViewModel(message.WorkItem);
                window.Show();

                _windows.Add(window);

            }
            else
            {
                if (tuple.Item1.WindowState == WindowState.Minimized)
                {
                    tuple.Item1.WindowState = WindowState.Normal;
                }
                tuple.Item1.Activate();
            }
        }

        private void HandleShowQueryMessage(ShowQueryMessage message)
        {

            var tuple = GetShellWindows<QueryViewModel>().FirstOrDefault(t => t.Item2.Model.Id == message.Query.Id);

            if (tuple == null)
            {
                var window = new RibbonShell();
                window.ViewModel = new QueryViewModel(message.Query);
                window.Show();
                _windows.Add(window);
            }
            else
            {
                if (tuple.Item1.WindowState == WindowState.Minimized)
                {
                    tuple.Item1.WindowState = WindowState.Normal;
                }
                tuple.Item1.Activate();
            }
        }

        private void HandleSimpleRequestMessage(Fissum.TrayApp.Messages.UserSimpleRequestMessage message)
        {
            if (message == null)
                throw new ArgumentNullException("message", "message is null.");

            var ret = MessageBox.Show(message.Message, message.Title, MessageBoxButton.YesNo);

            switch (ret)
            {
                case MessageBoxResult.Yes:
                    message.Response = true;
                    break;
                case MessageBoxResult.No:
                    message.Response = false;
                    break;
                default:
                    message.Response = null;
                    break;
            }
        }


        private void HandleOpenLinkInBrowseMessage(Uri uri)
        {
            var startInfo = new ProcessStartInfo(uri.ToString()) { Verb = "Open" };
            Process.Start(startInfo);
        }


        private void HandleShowAboutWindowMessage(ShowAboutWindowMessage message)
        {
            AboutWindow window = new AboutWindow();
            window.ShowDialog();
        }

        private void HandleShowDebugMessage(ShowDebugWindowMessage message)
        {
            var tuple = GetShellWindows<DebugViewModel>().FirstOrDefault();

            if (tuple == null)
            {
                var window = new RibbonShell();
                window.ViewModel = DebugViewModel.Current;
                window.Show();
                _windows.Add(window);
            }
            else
            {
                if (tuple.Item1.WindowState == WindowState.Minimized)
                {
                    tuple.Item1.WindowState = WindowState.Normal;
                }
                tuple.Item1.Activate();
            }
        }
        #endregion

        #region Media

        public Image CreateImage(string imageKey)
        {
            var uri = new Uri(string.Format("pack://application:,,,/Media/{0}.png", imageKey));
            var bitmapImage = new BitmapImage(uri);
            return new Image() { Source = bitmapImage };
        }

        public BitmapSource Favorite16Image { get; private set; }
        public BitmapSource NotFavorite16Image { get; private set; }
        public BitmapSource TFS16Image { get; private set; }
        public BitmapSource TFS16DisabledImage { get; private set; }
        public BitmapSource Refresh16Image { get; private set; }
        public BitmapSource Refresh32Image { get; private set; }
        public BitmapSource QueryTypeList16Image { get; private set; }
        public BitmapSource QueryTypeOneHop16Image { get; private set; }
        public BitmapSource QueryTypeTree16Image { get; private set; }
        public BitmapSource New16Image { get; private set; }
        public BitmapSource Folder16Image { get; private set; }
        public BitmapSource Info16Image { get; private set; }
        public BitmapSource Bug16Image { get; private set; }
        #endregion

        #region Shells management

        private List<IShellWindow> _windows;

        private IEnumerable<Tuple<Window, VIEWMODEL>> GetShellWindows<VIEWMODEL>() where VIEWMODEL : CoreViewModelBase
        {
            return from window in _windows
                   let typedViewModel = window.ViewModel as VIEWMODEL
                   let w = window as Window
                   where typedViewModel != null && w != null
                   select new Tuple<Window, VIEWMODEL>(w, typedViewModel);
        }
        internal void RemoveFromWindowList(IShellWindow window)
        {
            var w = window as Window;
            if (w == null)
            {
                throw new ArgumentException("Must be of type Window", "window");
            }

            _windows.Remove(window);
        }

        #endregion

        protected override void OnExit(ExitEventArgs e)
        {
            base.OnExit(e);
            SaveConfiguration();
        }
    }
}
