﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.TeamFoundation.Client;
using Fissum.Core.Configuration;
using Fissum.Core.Repository;

namespace Fissum.TrayApp.Lib
{
    internal class ConfigurationRepository
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(ConfigurationRepository));
        
        private GlobalConfiguration _configuration;
        private string _configurationFileName;        
        private object _syncObject = new object();
        private WindowConfigurationBase _mainWindowConfiguration;
        private RibbonConfiguration _ribbonConfiguration;

        internal ConfigurationRepository(string configurationFileName)
        {
            if (String.IsNullOrEmpty(configurationFileName))
                throw new ArgumentException("configurationFileName is null or empty.", "configurationFileName");

            _configurationFileName = configurationFileName;
            LoadConfiguration();
            _mainWindowConfiguration = new WindowConfigurationBase(_configuration, "MAIN", 300, 500);
            _ribbonConfiguration = new RibbonConfiguration(_configuration);
        }

        private bool LoadConfiguration()
        {
            lock (_syncObject)
            {
                try
                {
                    if (File.Exists(_configurationFileName))
                    {
                        var stream = File.OpenRead(_configurationFileName);
                        using (stream)
                        {
                            _configuration = GlobalConfiguration.Load(stream);
                        }
                        _configuration.AppendRegisteredTPC(false);
                    }
                    else
                    {
                        _configuration = new GlobalConfiguration();
                        _configuration.AppendRegisteredTPC(true);
                    }
                  
                    return true;
                }
                catch (Exception ex)
                {
                    _log.Error("Error while loading configuration", ex);
                    _configuration = new GlobalConfiguration();
                    _configuration.AppendRegisteredTPC(true);
                                   
                    return false;
                }
                
            }

        }

        public void SaveConfiguration()
        {
            lock (_syncObject)
            {
                var stream = File.OpenWrite(_configurationFileName);
                using (stream)
                {
                    _configuration.Save(stream);
                    stream.Flush();
                    stream.Close();
                }
            }
        }

        public List<TPCConfiguration> ActualTPCConfigList
        {
            get
            {
                return _configuration.TPCList;
            }
        }

        public GlobalConfiguration Configuration
        {
            get
            {
                return _configuration;
            }
        }


        public RibbonConfiguration RibbonConfiguration
        {
            get
            {
                return _ribbonConfiguration;
            }
        }

        public static ConfigurationRepository Current { get; private set; }
        public static void IntializeRepository(string configurationFileName)
        {
            Current = new ConfigurationRepository(configurationFileName);
        }
    }




    
}
