using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.TeamFoundation.Client;
using System.Xml.Linq;
using System.IO;
using Fissum.Core.Configuration;
using System.ComponentModel;

namespace Fissum.TrayApp.Lib
{
    public class WindowConfigurationBase
    {
        protected GlobalConfiguration Config { get; private set; }
        private string _windowName;
        private int _defaultWidth;
        private int _defaultHeight;

        /// <summary>
        /// Initializes a new instance of the WindowConfiguration class.
        /// </summary>
        /// <param name="config"></param>
        /// <param name="windowName"></param>
        /// <param name="defaultWidth"></param>
        /// <param name="defaultHeight"></param>
        public WindowConfigurationBase(GlobalConfiguration config, string windowName, int defaultWidth, int defaultHeight)
        {
            if (config == null)
                throw new ArgumentNullException("config", "config is null.");
            if (String.IsNullOrEmpty(windowName))
                throw new ArgumentException("windowName is null or empty.", "windowName");

            Config = config;
            _windowName = "WINDOW_" + windowName;
            _defaultWidth = defaultWidth;
            _defaultHeight = defaultHeight;
        }


        /// <summary>
        /// Initializes a new instance of the UIConfiguration class.
        /// </summary>
        /// <param name="config"></param>


        public int Width
        {
            get
            {
                return Config.GetParameterValue(_windowName, "width", _defaultWidth);
            }
            set
            {
                Config.SetParameterValue(_windowName, "width", value);
            }
        }

        public int Height
        {
            get
            {
                return Config.GetParameterValue(_windowName, "height", _defaultWidth);
            }
            set
            {
                Config.SetParameterValue(_windowName, "height", value);
            }
        }

    }
}