using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.UI.Core;
using System.Windows.Input;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using Fissum.TrayApp.Lib;
using System.Text.RegularExpressions;
using Microsoft.TeamFoundation.Client;
using Fissum.Core.Configuration;
using Fissum.UI.Core.Menu;
using GalaSoft.MvvmLight.Command;
using Fissum.TrayApp.Model;
using Fissum.TrayApp.Messages;
using Fissum.Core.Repository;
using Microsoft.Practices.Unity;
using GalaSoft.MvvmLight.Messaging;

namespace Fissum.TrayApp.ViewModel
{
    internal class TPCMenuItem : MenuItemBase
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(TPCMenuItem));

        MainViewModel _mainViewModel;
        SimpleMenuItem _connectMenuItem;
        SimpleMenuItem _connectAtStartupMenuItem;
        SimpleMenuItem _doNotConnectAtStartupMenuItem;
        SimpleMenuItem _refreshMenuItem;
        TPCState _currentState;
        /// <summary>
        /// Initializes a new instance of the TCPMenuItem class.
        /// </summary>
        public TPCMenuItem(TPCModel tpcModel, MainViewModel mainViewModel)
        {
            IsConnected = false;

            Icon = Program.Application.TFS16DisabledImage;
            _currentState = TPCState.Disconnected;
            TPCModel = tpcModel;
            _mainViewModel = mainViewModel;
            FillSubmenu();

            var messenger = UnityRepository.Container.Resolve<Messenger>();
            messenger.Register<TPCStateMessage>(this, tpcModel.Uri, HandleStateMessage);
        }

        private void ClearProjectMenu()
        {
            Items = new List<IMenuItem>();
            RaisePropertyChangedSafe(() => Items);
        }



        private void RefreshAutoConnectMenuItemVisibility()
        {
            _doNotConnectAtStartupMenuItem.Visible = TPCModel.AutoConnect;
            _connectAtStartupMenuItem.Visible = !TPCModel.AutoConnect;
        }

        private void FillSubmenu()
        {
            var newItemList = new List<IMenuItem>();

            //Adding projects
            if (TPCModel.HasAuthenticated)
            {
                newItemList.AddRange(
                    from pm in TPCModel.ProjectList
                    orderby pm.Name
                    select new ProjectMenuItem(pm, true));

                newItemList.Add(new SeparatorMenuItem());
            }


            ICommand autoConnectCommand = new RelayCommand(() => DoSwitchAutoConnect());


            _connectMenuItem = new SimpleMenuItem();
            _connectMenuItem.Header = "Connect";
            _connectMenuItem.IsEnabled = IsConnected;
            _connectMenuItem.Command = new RelayCommand(() => DoConnect());





            _connectAtStartupMenuItem = new SimpleMenuItem();
            _connectAtStartupMenuItem.Header = "Auto connect at startup";

            _connectAtStartupMenuItem.Command = autoConnectCommand;

            _doNotConnectAtStartupMenuItem = new SimpleMenuItem();
            _doNotConnectAtStartupMenuItem.Header = "Do not connect at startup";

            _doNotConnectAtStartupMenuItem.Command = autoConnectCommand;
            RefreshAutoConnectMenuItemVisibility();

            newItemList.Add(_connectMenuItem);
            newItemList.Add(_connectAtStartupMenuItem);
            newItemList.Add(_doNotConnectAtStartupMenuItem);

            Items = newItemList;

            RaisePropertyChangedSafe(() => Items);
        }


        private void HandleStateMessage(TPCStateMessage message)
        {
            if (message.State == _currentState)
                return;

            _currentState = message.State;
            switch (_currentState)
            {
                case Model.TPCState.Disconnected:
                    IsConnected = false;
                    IsUpdating = false;
                    break;
                case Model.TPCState.Connecting:
                    IsConnected = false;
                    IsUpdating = true;
                    break;
                case Model.TPCState.Updating:
                    IsConnected = true;
                    IsUpdating = true;
                    break;
                case Model.TPCState.UpToDate:
                    IsConnected = true;
                    IsUpdating = false;
                    break;
                case Model.TPCState.Dirty:
                default:
                    IsConnected = false;
                    IsUpdating = false;
                    break;

            }
            if (!IsUpdating)
            {
                FillSubmenu();
            }


            RaisePropertyChangedSafe(() => IsConnected);
            RaisePropertyChangedSafe(() => IsUpdating);
            _connectMenuItem.Visible = !IsConnected;

            if (IsConnected)
            {
                Icon = Program.Application.TFS16Image;
            }
            else
            {
                Icon = Program.Application.TFS16DisabledImage;
            }
        }

        private void DoConnect()
        {
            this.ProcessAsync(() => TPCModel.Connect());
        }

        private void DoSwitchAutoConnect()
        {
            TPCModel.AutoConnect = !TPCModel.AutoConnect;
            RefreshAutoConnectMenuItemVisibility();
        }


        public TPCModel TPCModel { get; private set; }


        public bool IsUpdating { get; private set; }
        public bool IsConnected { get; private set; }
    }

}