﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Fissum.Linq;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Test_Fissum.Linq
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class LinqOnTFSWi
    {
        private static TPCInstance _tfsinstance;


        private static TfsTeamProjectCollection TPC
        {
            get
            {
                return _tfsinstance.TeamProjectCollection;
            }
        }

        private static Project Project
        {
            get
            {
                return _tfsinstance.WorkItemStore.Projects[0];
            }
        }

        public LinqOnTFSWi()
        {

        }



        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [ClassInitialize()]
        public static void InitTFSConnection(TestContext testContext)
        {

            _tfsinstance = TPCInstance.ConnectToTFS(Contants.SERVER_URL);
            if (_tfsinstance == null)
            {
                throw new InvalidOperationException("Connection Failed to TFS");
            }
        }

        [ClassCleanup()]
        public static void EndTFSConnection()
        {
            if (_tfsinstance != null)
            {
                _tfsinstance.Dispose();
            }
        }

        [TestMethod]
        public void WI_simple_select()
        {
            var q = from wi in TPC.WorkItemSet()
                    select wi;

            var ret = q.ToList();
        }

        [TestMethod]
        public void WI_with_title_equal_search()
        {
            var q = from wi in TPC.WorkItemSet()
                    where wi.Title == "Set up: Configure Build"
                    select wi;

            var ret = q.ToList();
        }

        [TestMethod]
        public void WI_with_changed_by_me()
        {
            var q = from wi in TPC.WorkItemSet()
                    where wi.ChangedBy == QueryConstant.Me
                    select wi;

            var ret = q.ToList();
        }

        [TestMethod]
        public void WI_with_field_method()
        {
            var q = from wi in TPC.WorkItemSet()
                    where wi.Field<string>(SystemField.ChangedBy) == QueryConstant.Me
                    select wi;

            var ret = q.ToList();
        }


        [TestMethod]
        public void WI_simple_select_order_by()
        {
            var q = from wi in TPC.WorkItemSet()
                    orderby wi.Title
                    select wi;

            var ret = q.ToList();
        }

        [TestMethod]
        public void WI_simple_select_order_by_with_field_method()
        {
            var q = from wi in TPC.WorkItemSet()
                    orderby wi.Field<string>(SystemField.ChangedBy)
                    select wi;

            var ret = q.ToList();
        }

        
    }
}