using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.TeamFoundation.Client;
using System.Xml.Linq;
using System.IO;

namespace Fissum.Core.Configuration
{
    public class ProjectConfiguration
    {
        private ConfigurationDataSet.ProjectRow _data;
        private GlobalConfiguration _globalConfiguration;

        /// <summary>
        /// Initializes a new instance of the FavoriteProject class.
        /// </summary>
        /// <param name="data"></param>
        internal ProjectConfiguration(GlobalConfiguration globalConfiguration, ConfigurationDataSet.ProjectRow data)
        {
            _data = data;
            _globalConfiguration = globalConfiguration;
        }
        public string TFSName
        {
            get
            {
                return _data.Name;
            }
        }

        public string Name
        {
            get
            {
                if (string.IsNullOrWhiteSpace(_data.Alias))
                {
                    return _data.Name;
                }
                else
                {
                    return _data.Alias;
                }
            }
        }
        public string Alias
        {
            get
            {
                return _data.Alias;
            }
            set
            {
                _data.Alias = value.Trim(); ;
            }

        }

        public bool IsFavorite
        {
            get
            {
                return _data.IsFavorite;
            }
            set
            {
                _data.IsFavorite = value;
            }
        }

        public bool IsIgnored
        {
            get
            {
                return _data.IsIgnored;
            }
            set
            {
                _data.IsIgnored = value;
            }
        }

        public Uri TFSUri
        {
            get
            {
                return new Uri(_data.TPCUri);
            }
        }

        public System.Windows.Media.Color? Color
        {
            get
            {
                if (_data.IsColorNull())
                {
                    return null;
                }
                else
                {
                    return System.Windows.Media.Color.FromRgb((byte)(_data.Color >> 16 & 0xff), 
                                                              (byte)(_data.Color >> 8 & 0xff), 
                                                              (byte)(_data.Color & 0xff));
                }
            }
            set
            {
                if (value == null)
                {
                    _data.SetColorNull();
                }
                else
                {
                    
                    var color = value.Value;
                    _data.Color = (uint)(color.R << 16) | (uint)(color.G << 8) | (uint)(color.B);
                }
            }
        }
    }
}
