using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.TeamFoundation.Client;
using System.Xml.Linq;
using System.IO;
using System.Diagnostics;

namespace Fissum.Core
{
    public class WeakReference<T> where T:class
    {
        WeakReference _weakRef;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:WeakReference&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="target">The target.</param>
        [DebuggerStepThrough]
        public WeakReference(T target)
        {
            _weakRef = new WeakReference(target);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:WeakReference&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="target">The target.</param>
        /// <param name="trackResurrection">if set to <c>true</c> [track resurrection].</param>
        [DebuggerStepThrough]
        public WeakReference(T target, bool trackResurrection)
        {
            _weakRef = new WeakReference(target, trackResurrection);
        }

        /// <summary>
        /// Gets a value indicating whether this instance is alive.
        /// </summary>
        /// <value><c>true</c> if this instance is alive; otherwise, <c>false</c>.</value>
        public bool IsAlive
        {
            [DebuggerStepThrough]
            get { return _weakRef.IsAlive; }
        }

        /// <summary>
        /// Gets or sets the target.
        /// </summary>
        /// <value>The target.</value>
        public T Target
        {
            [DebuggerStepThrough]
            get { return (T)_weakRef.Target; }
            [DebuggerStepThrough]
            set { _weakRef.Target = value; }
        }

        /// <summary>
        /// Gets a value indicating whether [track resurrection].
        /// </summary>
        /// <value><c>true</c> if [track resurrection]; otherwise, <c>false</c>.</value>
        public bool TrackResurrection
        {
            [DebuggerStepThrough]
            get { return _weakRef.TrackResurrection; }
        }
    }
}
