using System;
using System.Linq;
using System.Collections.Generic;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace Fissum.Models
{
    [DebuggerDisplay("{Name}")]
    public class WorkItemTypeModel
    {
        private WorkItemType _type;
        private ProjectModel _projectModel;
        private Lazy<List<WorkItemFieldDefinitionModel>> _fields;

        /// <summary>
        /// Initializes a new instance of the WorkItemTypeModel class.
        /// </summary>
        /// <param name="projectModel"></param>
        /// <param name="type"></param>
        public WorkItemTypeModel(ProjectModel projectModel, WorkItemType type)
        {

            _type = type;
            _projectModel = projectModel;
            _fields = new Lazy<List<WorkItemFieldDefinitionModel>>(() =>
                {
                    return (from field in type.FieldDefinitions.Cast<FieldDefinition>()
                            select new WorkItemFieldDefinitionModel()
                            {
                                IsReadOnly = (!field.IsEditable) || field.IsComputed,
                                Name = field.Name,
                                ReferenceName = field.ReferenceName,
                                TypeName = field.SystemType,
                                PropertyName = Regex.Replace(field.Name, @"\W", String.Empty),
                                Description = field.HelpText
                             

                            }).ToList();

                });
        }


        public string Name
        {
            get
            {
                return _type.Name;
            }
        }
 
        public string Description
        {
            get
            {
                return _type.Description;
            }
        }

        internal WorkItemType Type
        {
            get
            {
                return _type;
            }
        }

        public ProjectModel ProjectModel
        {
            get
            {
                return _projectModel;
            }
        }

        public List<WorkItemFieldDefinitionModel> Fields
        {
            get
            {
                return _fields.Value;
            }
        }
    }
}
