﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Fissum.Models;

namespace Fissum.TrayApp.Control
{
    /// <summary>
    /// Interaction logic for QueryControl.xaml
    /// </summary>
    internal partial class QueryControl : UserControl
    {
        public static readonly DependencyProperty QueryModelProperty = DependencyProperty.Register("QueryModel", typeof(QueryModel), typeof(QueryControl), new UIPropertyMetadata(null, new PropertyChangedCallback(OnQueryModelChanged), new CoerceValueCallback(OnCoerceQueryModel)));

        private static object OnCoerceQueryModel(DependencyObject o, object value)
        {
            QueryControl queryControl = o as QueryControl;
            if (queryControl != null)
                return queryControl.OnCoerceQueryModel((QueryModel)value);
            else
                return value;
        }

        private static void OnQueryModelChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            QueryControl queryControl = o as QueryControl;
            if (queryControl != null)
                queryControl.OnQueryModelChanged((QueryModel)e.OldValue, (QueryModel)e.NewValue);
        }

        protected virtual QueryModel OnCoerceQueryModel(QueryModel value)
        {
            // TODO: Keep the proposed value within the desired range.
            return value;
        }

        protected virtual void OnQueryModelChanged(QueryModel oldValue, QueryModel newValue)
        {
            hg.HierarchicalGridDataSource = newValue;
        }

        public QueryModel QueryModel
        {
            // IMPORTANT: To maintain parity between setting a property in XAML and procedural code, do not touch the getter and setter inside this dependency property!
            get
            {
                return (QueryModel)GetValue(QueryModelProperty);
            }
            set
            {
                SetValue(QueryModelProperty, value);
            }
        }

        public static readonly DependencyProperty CurrentSelectedWorkItemProperty = DependencyProperty.Register("CurrentSelectedWorkItem", typeof(WorkItemModel), typeof(QueryControl), new UIPropertyMetadata(null));

        public WorkItemModel CurrentSelectedWorkItem
        {
            // IMPORTANT: To maintain parity between setting a property in XAML and procedural code, do not touch the getter and setter inside this dependency property!
            get
            {
                return (WorkItemModel)GetValue(CurrentSelectedWorkItemProperty);
            }
            set
            {
                SetValue(CurrentSelectedWorkItemProperty, value);
            }
        }

        public static readonly DependencyProperty SelectedWorkItemsProperty = DependencyProperty.Register("SelectedWorkItems", typeof(List<WorkItemModel>), typeof(QueryControl), new UIPropertyMetadata(null));

        public List<WorkItemModel> SelectedWorkItems
        {
            // IMPORTANT: To maintain parity between setting a property in XAML and procedural code, do not touch the getter and setter inside this dependency property!
            get
            {
                return (List<WorkItemModel>)GetValue(SelectedWorkItemsProperty);
            }
            set
            {
                SetValue(SelectedWorkItemsProperty, value);
            }
        }

        public new event EventHandler MouseDoubleClick;

        #region OnMouseDoubleClick
        /// <summary>
        /// Triggers the MouseDoubleClick event.
        /// </summary>
        public virtual void OnMouseDoubleClick()
        {
            if (MouseDoubleClick != null)
                MouseDoubleClick(this,EventArgs.Empty);
        }
        #endregion
    


        public QueryControl()
        {
            InitializeComponent();
            hg.SelectedItemChanged+=new EventHandler(hg_SelectedItemChanged);
            hg.MouseDoubleClick += new EventHandler(hg_MouseDoubleClick);
         
        }

        void hg_MouseDoubleClick(object sender, EventArgs e)
        {
            OnMouseDoubleClick();
        }

        void hg_SelectedItemChanged(object sender, EventArgs e)
        {
            var queryItem = hg.CurrentSelectedItem as QueryModel.QueryModelItem;
            if (queryItem == null)
            {
                CurrentSelectedWorkItem = null;
            }
            else
            {
                CurrentSelectedWorkItem = queryItem.WorkItemModel;
            }
            if (hg.SelectedItems != null)
            {
                var selectedItems = hg.SelectedItems.Cast<QueryModel.QueryModelItem>();
                SelectedWorkItems = selectedItems.Select(qmi => qmi.WorkItemModel).ToList();
            }
            else
            {
                SelectedWorkItems = null;
            }
            
        }

   





    }
}
