using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Windows.Threading;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Diagnostics;

namespace Fissum.UI.Core
{
    public class AsyncViewModelBase : CoreViewModelBase
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(AsyncViewModelBase));
        

        private int _asyncTaskCount;
        private bool _isLoading;
        public bool IsLoading
        {
            get
            {
                return _isLoading;
            }
        }

        public event EventHandler IsLoadingChanged;

        public void ExecuteAsync<CONTEXT, PARAMETER>(CONTEXT context,
            Func<PARAMETER, CONTEXT,bool> executeAction,
            Action<ExecutionResult, CONTEXT> onExecutedAction,
            PARAMETER typedParameter)
            where CONTEXT : new()
        {
            
            AppendNewLoadingState(true);

            ProcessAsync(() =>
            {
                var result = new ExecutionResult();
                try
                {

                    var ret = executeAction(typedParameter, context);
                    result.Completed = ret;
                }
                catch (Exception ex)
                {
                    _log.Error("Error while executing async action", ex);
                    
                    result.Completed = false;
                    result.Exception = ex;
                }
                finally
                {

                    ProcessOnUIThread(() =>
                    {
                        onExecutedAction(result, context);
                        AppendNewLoadingState(false);
                    });
                }

            });
        }

   

        public AsyncViewModelBase()
     
        {

        }

        protected internal void AppendNewLoadingState(bool IsLoading)
        {

            CheckIfInDispatcherThread();
            if (IsLoading)
            {
                Interlocked.Increment(ref _asyncTaskCount);
                _isLoading = true;
            }
            else
            {
                var ret = Interlocked.Decrement(ref _asyncTaskCount);
                _isLoading = ret > 0;
            }
        }
    }
}
