using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Windows.Threading;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Diagnostics;
using GalaSoft.MvvmLight.Threading;
using GalaSoft.MvvmLight.Messaging;

namespace Fissum.UI.Core
{
    public class CoreViewModelBase : GalaSoft.MvvmLight.ViewModelBase
    {

        public bool Initialized { get; private set; }



        protected void CheckIfInDispatcherThread()
        {

            DispatcherHelper.UIDispatcher.VerifyAccess();
            
        }

        protected virtual void InitializeCore()
        {

        }

        public void Initialize()
        {
            if (!Initialized)
            {
                InitializeCore();
                Initialized = true;
            }
        }

        protected CoreViewModelBase()
        {

        }



        public void ProcessAsync(Action action)
        {
            Task.Factory.StartNew(() => action());
        }

        public void ProcessOnUIThread(Action action)
        {
            if (DispatcherHelper.UIDispatcher.CheckAccess())
            {
                action();
            }
            else
            {
                DispatcherHelper.UIDispatcher.BeginInvoke(action);
            }
        }


        protected void RaisePropertyChangedSafe(string propertyName)
        {

            if (String.IsNullOrEmpty(propertyName))
                throw new ArgumentException("propertyName is null or empty.", "propertyName");

            if (DispatcherHelper.UIDispatcher.CheckAccess())
            {
                RaisePropertyChanged(propertyName);
            }
            else
            {
                DispatcherHelper.UIDispatcher.BeginInvoke(new Action(() => RaisePropertyChanged(propertyName)));
            }
        }

        protected void RaisePropertyChangedSafe<T>(Expression<Func<T>> propertyExpression)
        {
            var propEx = propertyExpression.Body as System.Linq.Expressions.MemberExpression;
            if (propEx == null)
            {
                throw new ArgumentException("propertyExpression");
            }


            var propName = propEx.Member.Name;
            RaisePropertyChangedSafe(propName);
        }
    }

}
