using System;
using System.ComponentModel;
using Fissum.Core;

namespace Fissum.UI.Core.HG
{
    public class GridItemDataPropertyDescriptor : PropertyDescriptor
    {
        ColumnInfo _columnInfo;
        public GridItemDataPropertyDescriptor(ColumnInfo columnInfo)
            : base(columnInfo.DataProperty, null)
        {
            _columnInfo = columnInfo;
        }


        public override bool CanResetValue(object component)
        {
            return true;
        }

        public override Type ComponentType
        {
            get { return typeof(GridItemData); }
        }

        public override object GetValue(object component)
        {
            GridItemData data = component as GridItemData;

            var rawValue = data.GridItem.ItemSource.GetPropertyValue(_columnInfo.PropertyName);

            if (rawValue == null)
            {
                return string.Empty;
            }
            if (rawValue is string)
            {
                return rawValue;
            }
            else if (rawValue is int)
            {
                return ((int)rawValue).ToString();
            }
            else if (rawValue is DateTime)
            {
                return ((DateTime)rawValue).ToString("d");
            }
            else
            {
                return rawValue.ToString();
            }

        }

        public override bool IsReadOnly
        {
            get { return true; }
        }

        public override Type PropertyType
        {
            get { return typeof(string); }
        }

        public override void ResetValue(object component)
        {

        }

        public override void SetValue(object component, object value)
        {

        }

        public override bool ShouldSerializeValue(object component)
        {
            return false;
        }
    }
}
