using System;
using System.Collections.Generic;
using System.Linq;
using Fissum.Models;
using Fissum.UI.Core.TreeView;

namespace Fissum.UI.Core.ViewModel
{
    public sealed class QueryFolderTreeViewItemViewModel : TreeViewItemBase, ISelectable
    {
        private QueryFolderModel _queryFolderModel;
        private bool _selected;
        private bool _showSelected;

        public QueryFolderTreeViewItemViewModel(QueryFolderModel queryFolderModel, TreeViewItemBase parent,bool selected,bool showSelected)
            : base(parent, true)
        {
            if (queryFolderModel == null)
                throw new ArgumentNullException("folderModel", "folderModel is null.");
            _queryFolderModel = queryFolderModel;
            _selected = selected;
            _showSelected = showSelected;
        }

        public override bool? HasChildren
        {
            get
            {
                return _queryFolderModel.QueryList.Any() || _queryFolderModel.SubFolderList.Any();
            }
        }

        protected override void LoadChildren()
        {
            base.LoadChildren();

            var qf = _queryFolderModel.SubFolderList.Select(folder => new QueryFolderTreeViewItemViewModel(folder, this,_selected,_showSelected) as TreeViewItemBase);
            var qq = _queryFolderModel.QueryList.Select(query => new QueryTreeViewItemViewModel(query, this,_selected,_showSelected) as TreeViewItemBase);

            foreach (var item in qf.Union(qq))
            {
                Children.Add(item);
            }
        }

        private void UpdateChildrenSelectedState()
        {
            foreach (var item in Children.OfType<ISelectable>())
            {
                item.Selected = _selected;
            }
        }

        public string FolderName
        {
            get
            {
                return _queryFolderModel.Name;
            }
        }

        public bool ShowSelected
        {
            get
            {
                return _showSelected;
            }
            set
            {
                if (_showSelected == value)
                    return;
                _showSelected = value;
                RaisePropertyChangedSafe(() => ShowSelected);
            }
        }

        public bool Selected
        {
            get
            {
                return _selected;
            }
            set
            {
                if (_selected == value)
                    return;
                _selected = value;
                RaisePropertyChangedSafe(() => Selected);
                UpdateChildrenSelectedState();
            }
        }

    }
}
