using System;
using System.Collections.Generic;
using System.Linq;
using Fissum.Models;
using Fissum.UI.Core.TreeView;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
namespace Fissum.UI.Core.ViewModel
{
    public sealed class QueryTreeViewItemViewModel : TreeViewItemBase, ISelectable
    {
        private readonly QueryModel _queryModel;
        private bool _selected;
        private bool _showSelected;


        public QueryTreeViewItemViewModel(QueryModel queryModel, TreeViewItemBase parent, bool selected, bool showSelected)
            : base(parent, true)
        {
            if (queryModel == null)
                throw new ArgumentNullException("folderModel", "folderModel is null.");
            _queryModel = queryModel;
            _selected = selected;
            _showSelected = showSelected;

            switch (queryModel.Definition.QueryType)
            {                           
                case QueryType.OneHop:
                    Type = "OneHop";
                    break;
                case QueryType.Tree:
                    Type = "Tree";
                    break;
                case QueryType.List:
                default:
                    Type = "List";   
                    break;
        
            }
        }
        public override bool? HasChildren
        {
            get
            {
                return false;
            }
        }

        public string QueryName
        {
            get
            {
                return _queryModel.Name;
            }
        }

        public string Type
        {
            get;
            private set;
        }

        public bool ShowSelected
        {
            get
            {
                return _showSelected;
            }
            set
            {
                if (_showSelected == value)
                    return;
                _showSelected = value;
                RaisePropertyChangedSafe(() => ShowSelected);
                
            }
        }

        public bool Selected
        {
            get
            {
                return _selected;
            }
            set
            {
                if (_selected == value)
                    return;
                _selected = value;
                RaisePropertyChangedSafe(() => Selected);               
            }
        }

        public QueryModel QueryModel
        {
            get
            {
                return _queryModel;
            }
        }
    }

}
