﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.Models;
using Fissum.UI.Core.TreeView;

namespace Fissum.UI.Core.ViewModel
{
    public sealed class SelectQueryDialogViewModel : CoreViewModelBase, IDialogViewModel
    {
        private ProjectModel _projectModel;

        #region IDialogViewModel Members

        public string DialogTitle
        {
            get { return "Select a query"; }
        }

        public object DialogIcon
        {
            get { return null; }
        }

        public IDialogWindow DialogWindow
        {
            get;
            set;
        }

        public List<DialogAction> Actions
        {
            get { return new List<DialogAction>(); }
        }

        #endregion


        /// <summary>
        /// Initializes a new instance of the SelectQueryDialogViewModel class.
        /// </summary>
        /// <param name="projectModel"></param>
        public SelectQueryDialogViewModel(ProjectModel projectModel, bool selected, bool showSelected )
        {
            _projectModel = projectModel;

            QueryFolders =_projectModel
                .QueryRootFolder
                .SubFolderList
                .Select(q => new QueryFolderTreeViewItemViewModel(q, null, selected, showSelected) as TreeViewItemBase)
                .ToList();
                
         


        }

        public IEnumerable<TreeViewItemBase> QueryFolders { get; private set; }


        private IEnumerable<QueryModel> GetSelectedQueries(IEnumerable<TreeViewItemBase> tvItems)
        {
            return tvItems
                .OfType<QueryTreeViewItemViewModel>()
                .Where(_ => _.IsSelected).Select(_ => _.QueryModel)
                .Union(GetSelectedQueries(tvItems.OfType<QueryFolderTreeViewItemViewModel>()));
        }

        public List<QueryModel> GetSelectedQueries()
        {
            return GetSelectedQueries(QueryFolders).ToList();
        }
    }
}