using System;
using Fissum.UI.Core;
using System.Text.RegularExpressions;
using System.Windows;

namespace Fissum.UI.Core
{
    public class ViewModelToViewDataTemplateSelector : System.Windows.Controls.DataTemplateSelector
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(ViewModelToViewDataTemplateSelector));
        
        public override System.Windows.DataTemplate SelectTemplate(object item, System.Windows.DependencyObject container)
        {
            if (!(item is CoreViewModelBase))
            {
                return null;
            }

            string className = item.GetType().FullName;

            var reg = new Regex(@"^(?<namespace>[\w\.]+)\.ViewModel\.(?<name>\w+)ViewModel$");

            var match = reg.Match(className);

            if (!match.Success)
            {
                _log.ErrorFormat("View '{0}' did not match view name pattern", className);
                return null;
            }

            string viewClassName = String.Format("{0}.View.{1}View", match.Groups["namespace"].Value, match.Groups["name"].Value);
            try
            {
                var viewType = item.GetType().Assembly.GetType(viewClassName, false);          
                if (viewType == null)
                {
                    _log.ErrorFormat("Cannot find view type '{0}' in assembly '{1}'", viewClassName,item.GetType().Assembly.FullName);
                    return null;
                }

                var dataTemplate = new DataTemplate();
                dataTemplate.VisualTree = new FrameworkElementFactory(viewType);

                return dataTemplate;
            }
            catch (Exception ex)
            {
                _log.Error(String.Format("Error while loading view type from viewmodel '{0}'", className), ex);
            }
            return null;
        }

    }
}
