﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;

namespace Test_Fissum.WIQL
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class TestParser
    {


        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void Test_Select_WithOneField()
        {
            var query = "SELECT [System.ID] from Workitems";           
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_WithTwoFields()
        {
            var query = "SELECT [System.ID],[System.CreatedBy] from Workitems";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(2, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
            Assert.AreEqual("System.CreatedBy", output.Fields[1]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test()
        {
            var query = "SELECT [System.ID] from Workitems where [System.ID] > 1";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Parens()
        {
            var query = "SELECT [System.ID] from Workitems where ( [System.ID] > 1 )";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_Two_Tests()
        {
            var query = "SELECT [System.ID] from Workitems where [System.ID] > 1 AND [System.ID] < 3";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Parameter_Value()
        {
            
            var query = "SELECT [System.ID] from Workitems where [System.Project] = @MyProject";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Parameter_String_With_Quote()
        {

            var query = "SELECT [System.ID] from Workitems where [System.Project] = 'He''llo'";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Parameter_String_With_DoubleQuote()
        {

            var query = "SELECT [System.ID] from Workitems where [System.Project] = \"He\"\"llo\"";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Today()
        {

            var query = "SELECT [System.ID] from Workitems where [System.CreateDate] = @today";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Me()
        {

            var query = "SELECT [System.ID] from Workitems where [System.CreateDate] = @me";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }


        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Project()
        {

            var query = "SELECT [System.ID] from Workitems where [System.CreateDate] = @project";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        public void Test_Select_Where_With_One_Test_And_Order_By_One_Field()
        {

            var query = "SELECT [System.ID] from Workitems where [System.CreateDate] = @project order by [System.ID]";
            var output = Fissum.Wiql.Parser.Process(query);
            Assert.AreEqual(1, output.Fields.Count);
            Assert.AreEqual("System.ID", output.Fields[0]);
        }

        [TestMethod]
        [DeploymentItem("ParseData.txt")]
        public void Test_ParseDataFile()
        {
            using (var reader = new StringReader(File.ReadAllText("ParseData.txt")))
            {
                string line = null;
                while ((line = reader.ReadLine()) != null)
                {
                    try
                    {
                        var output = Fissum.Wiql.Parser.Process(line);
                    }
                    catch (Exception ex)
                    {
                        Assert.Fail(string.Format("'{0}' cannot be parsed", line));
                    }
                }
            }
        }

        [TestMethod]
        public void Test_Query_In_Statement()
        {

            var query =     @"SELECT [System.Id], [System.WorkItemType], [Microsoft.VSTS.Common.Rank], [System.State], [System.AssignedTo], [System.Title] FROM WorkItems WHERE [System.TeamProject] = @project AND [System.State] IN ('Active', 'Pending', 'Proposed', 'Requested')  ORDER BY [Microsoft.VSTS.Common.Rank], [System.WorkItemType], [System.Id]";
  //          var query = @"select [System.Id], [System.Links.LinkType], [System.Title] from WorkItemLinks  where Source.[System.TeamProject] = @project  mode (DoesNotContain)"; // where (Source.[System.TeamProject] = @project and Source.[System.WorkItemType] = 'User Story') and Source.[System.IterationPath] under 'AgileProject\Iteration 1' and [System.Links.LinkType] <> '' and Target.[System.WorkItemType] = 'Test Case' order by [System.Id]";
           
            var output = Fissum.Wiql.Parser.Process(query);
        
        }
        //
    }
}
