﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.TeamSystem;
using Fissum.Core;
using System.Threading;

namespace Demo.CustomQuery
{
    class Program
    {
        static void Main(string[] args)
        {
            Fissum.Core.Context context = new Fissum.Core.Context(ContextMode.Normal);

            context.ContextEvent += new EventHandler<Fissum.Core.ContextGlobalEvent>(ContextEventHandler);
            AutoResetEvent waitHandle = new AutoResetEvent(false);

            context.ConnectAsync(() => waitHandle.Set());

            waitHandle.WaitOne();

            Console.WriteLine("Initialisation done.");

            TeamFoundationServer server = (from TeamFoundationServer srv in context.Servers where srv.Connected select srv).FirstOrDefault();
            if (server == null)
            {
                Console.WriteLine("No connected server to test Linq capabilities");
                Quit();
            }
            Console.WriteLine("Working on {0}", server.Uri);


            while (true)
            {
                Console.WriteLine("Enter your query (type 'exit' to quit):");
                Console.Write(">");
                string wiql = Console.ReadLine();

                try
                {
                    WorkItemQuery query = new WorkItemQuery(server, null,"custom query", wiql,null);


                    var wiArray = query.GetWorkItems();

                    ShowWorkItems(wiArray);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Exception '{0}' thrown:", ex.GetType().ToString());
                    Console.WriteLine(ex.Message);
                    Console.ReadLine();
                    Console.WriteLine(ex.StackTrace);
                }
            }            
        }

        static void ShowWorkItems(IEnumerable<WorkItemObject> wiEnum)
        {
            foreach (var wi in wiEnum)
            {
                Console.WriteLine("#{0}: '{1}'", wi.Id, wi.Title);
            }
        }


        static void ContextEventHandler(object sender, Fissum.Core.ContextGlobalEvent e)
        {
            Console.WriteLine("{0}/:{1}: {2}", e.Level, e.Type, e.Message);
        }

        private static void Quit()
        {
            Console.WriteLine("<Enter To Exit>");
            Console.ReadLine();
            Environment.Exit(0);
        }
    }
}
