﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using Fissum.Core;
using Fissum.Core.Linq;
using Fissum.TeamSystem;

namespace Demo.Linq
{
    class Program
    {
        private static void Quit()
        {
            Console.WriteLine("<Enter To Exit>");
            Console.ReadLine();
            Environment.Exit(0);
        }
        static void Main(string[] args)
        {
            Fissum.Core.Context context = new Fissum.Core.Context(ContextMode.Normal);
          
            context.ContextEvent += new EventHandler<Fissum.Core.ContextGlobalEvent>(ContextEventHandler);
            AutoResetEvent waitHandle = new AutoResetEvent(false);

            context.ConnectAsync(() => waitHandle.Set());

            waitHandle.WaitOne();
         
            TeamFoundationServer server = null;
          
            server = (from TeamFoundationServer srv in context.Servers where srv.Connected select srv).FirstOrDefault();
            if (server == null)
            {
                Console.WriteLine("No connected server to test Linq capabilities");
                Quit();               
            }

            Console.WriteLine("Working on ['{0}']", server.Uri);

            string searchString = "work";

            var q = (from wi in server.WorkItems()
                     where
                        wi.Title.Contains(searchString) ||
                        wi.History.Contains(searchString) ||
                        wi.Description.Contains(searchString) &&
                        wi.Field<string>(SystemField.AssignedTo) == QueryConstant.Me
                     orderby wi.CreatedDate descending, wi.Title
                     select wi
                     ).ToList();



            ShowWorkItems(q);

            Quit();
        }

        static string LimitString(string str, int maxLenght)
        {
            if (str == null)
            {
                return string.Empty;
            }
            if (str.Length < maxLenght)
            {
                return str;
            }
            return str.Substring(0, maxLenght) + "...";
        }

        static void ShowWorkItems(IEnumerable<WorkItemObject> wiEnum)
        {
            foreach (var wi in wiEnum)
            {
                Console.WriteLine(
                    @"------------------------------------------------------------------------
#{0}: '{1}, {3:d}'
{2}", 
                    wi.Id, 
                    wi.Title, 
                    LimitString(wi.Description,50),
                    wi.CreatedDate);
            }
        }
        

        static void ContextEventHandler(object sender, Fissum.Core.ContextGlobalEvent e)
        {
            Console.WriteLine("{0}/:{1}: {2}", e.Level, e.Type, e.Message);        
        }
    }
}
