using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Fissum.TeamSystem;
using Fissum.Lib;
using Fissum.Core;

namespace Fissum
{

    internal class ProjectToolStripMenuItem: ToolStripMenuItem
    {
        private Project _project;
        ConfigurationDataSet.ProjectTableRow _configuration;


        /// <summary>
        /// Rebuilds the menu.
        /// </summary>
        public void RebuildMenu()
        {

            DropDownItems.Clear();
            if (Project == null)
            {
                Image = GlobalRessources.Warning;
                return;
            }
            
            if (!_project.Server.Connected)
            {
                Image = GlobalRessources.DeleteHS;
                return;
            }

            ConfigurationDataSet.QueryTableRow[] queryRows = null;
            ConfigurationDataSet.WITypeTableRow[] wiRows = null;

            if (_configuration.FilterQueries)
            {
                queryRows = _configuration.GetQueryTableRows();
                
            }
            if (_configuration.FilterWorkItems)
            {
                wiRows = _configuration.GetWITypeTableRows();
            }

            int queryNumber = 0;

            if (_configuration.IsQueryTreeSeparatorNull() || String.IsNullOrEmpty(_configuration.QueryTreeSeparator))
            {

                foreach (WorkItemQuery query in _project.Queries)
                {
                    if (_configuration.FilterQueries)
                    {
                        bool selected = false;

                        foreach (ConfigurationDataSet.QueryTableRow row in queryRows)
                        {
                            if ((row.QueryName == query.Name) && (row.IsTeamQuery == query.IsTeamQuery))
                            {
                                selected = true;
                                break;
                            }
                        }

                        if (!selected)
                        {
                            continue;
                        }
                    }

                    queryNumber++;
                    WorkItemQueryToolStripMenuItem wqStrip = new WorkItemQueryToolStripMenuItem(query);
                    DropDownItems.Add(wqStrip);
                    Application.DoEvents();

                }
            }
            else
            {
                QueryNodeMenuItem rootNode = new QueryNodeMenuItem("");

                foreach (WorkItemQuery query in _project.Queries)
                {
                    string[] nameList = query.Name.Split(new char[] {_configuration.QueryTreeSeparator[0] } , StringSplitOptions.RemoveEmptyEntries);
                    if ((nameList == null) ||
                        (nameList.Length == 0))
                    {
                        continue;
                    }

                    rootNode.AttachQuery(query, new List<string>(nameList));
                }
                rootNode.BuildMenu(this);
            }




            int wtypeNumber = 0;         
            foreach (WorkItemType type in _project.Types)
            {
                if (_configuration.FilterWorkItems)
                {
                    bool selected = false;

                    foreach (ConfigurationDataSet.WITypeTableRow row in wiRows)
                    {
                        if (row.WITypeName == type.Name)
                        {
                            selected = true;
                            break;
                        }
                    }

                    if (!selected)
                    {
                        continue;
                    }                   
                }

                if ((wtypeNumber == 0) && (queryNumber > 0))
                {
                    DropDownItems.Add(new ToolStripSeparator());
                }
                wtypeNumber++;

                WorkItemTypeToolStripMenuItem wtStrip = new WorkItemTypeToolStripMenuItem(type);
                DropDownItems.Add(wtStrip);
                Application.DoEvents();
            }

            foreach (WorkItemModel model in WorkItemModel.GetModels(Fissum.FissumApplication.Context,_project))
            {
                WorkItemModelToolStripMenuItem wmStrip = new WorkItemModelToolStripMenuItem(model);
                DropDownItems.Add(wmStrip);

                Application.DoEvents();
            }


            if ((_project.AreaNodes.Length > 0) ||
                (_project.IterationNodes.Length > 0))
            {
                DropDownItems.Add(new ToolStripSeparator());
            }


            if (_project.AreaNodes.Length > 0)
            {
                ToolStripMenuItem menuItem = new ToolStripMenuItem("Area Manager...");
                menuItem.Click += delegate(object sender, EventArgs e)
                {
                    NodeManagementForm form;

                    form = new NodeManagementForm(_project, NodeManagementForm.NodeType.Area);
                    form.Show();
                };

                DropDownItems.Add(menuItem);
            }

            if (_project.IterationNodes.Length > 0)
            {
                ToolStripMenuItem menuItem = new ToolStripMenuItem("Iteration Manager...");
                menuItem.Click += delegate(object sender, EventArgs e)
                {
                    NodeManagementForm form;

                    form = new NodeManagementForm(_project, NodeManagementForm.NodeType.Iteration);
                    form.Show();
                };
                DropDownItems.Add(menuItem);
            }
            DropDownItems.Add(new ToolStripSeparator());
            ProjectWssSiteToolStripMenuItem wssMenuItem = new ProjectWssSiteToolStripMenuItem(_project);
            DropDownItems.Add(wssMenuItem);
        }

        void GotoTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {

            }
        }





        /// <summary>
        /// Initializes a new instance of the <see cref="T:ProjectToolStripMenuItem"/> class.
        /// </summary>
        /// <param name="project">The project.</param>
        /// <param name="alias">The alias.</param>
        public ProjectToolStripMenuItem(Project project, ConfigurationDataSet.ProjectTableRow configuration)
            : base()
        {
            _project = project;
                _configuration = configuration;

            Text = _configuration.Name;

            try
            {
                _project.Load();
            }
            catch
            {
                _project = null;
            }
            RebuildMenu();
        }

        /// <summary>
        /// Gets the project.
        /// </summary>
        /// <value>The project.</value>
        public Project Project
        {
            get
            {
                return _project;
            }
        }

        /// <summary>
        /// Gets the alias.
        /// </summary>
        /// <value>The alias.</value>
        public string Alias
        {
            get
            {
                return _configuration.Name;
            }
        }
    }
}
