using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using Fissum.Core;
using Fissum.TeamSystem;
using System.Windows.Markup;

namespace Fissum.Core.Configuration
{
    /// <summary>
    /// Manual configuration for Fissum context
    /// </summary>
    public class CoreConfiguration
    {
        public ServerInfoList ServerList { get; set; }
        public bool AllowUI { get; set; }
        public bool MergeWithTeamExplorerServers { get; set; }

        public CoreConfiguration()
        {
            ServerList = new ServerInfoList();
        }

        internal static CoreConfiguration LoadFromDisk(string path)
        {
            CoreConfiguration result;

            if (!File.Exists(path))
            {
                return null;
            }

            try
            {
                Stream stream = File.OpenRead(path);
                result = XamlReader.Load(stream) as CoreConfiguration;
            }
            catch
            {
                result = null;
            }          
            return result;
        }

        internal static CoreConfiguration LoadFromDiskOrCreateWithFullServerList(string path,bool allowUI)
        {
            var result = LoadFromDisk(path);
            if (result == null)
            {
                result = new CoreConfiguration() { AllowUI = allowUI };
                List<Uri> servers = Fissum.TeamSystem.TeamFoundationServer.GetAvailableServers();
                foreach (var srv in servers)
                {
                    result.ServerList.Add(new ServerInfo()
                    {
                        ServerAddress= srv.ToString(),
                        AutoConnect = true

                    });
                }
            }
            return result;
        }
        /// <summary>
        /// Save the configuration to disk
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        internal bool SaveToDisk(string path)
        {
            try
            {
                Stream stream = File.OpenWrite(path);

                XamlWriter.Save(this, stream);
                return true;
            }
            catch
            {
                return false;
            }          
        }
    }
}