using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using Fissum.Core;
using Fissum.TeamSystem;

namespace Fissum.Core
{
    public class ServerInfo: IEquatable<ServerInfo>
    {
        public string ServerAddress { get; set; }
        public string EncryptedLogin { get; set; }
        public string EncryptedPassword { get; set; }
        public bool AutoConnect { get; set; }

        public bool HasCustomCredential
        {
            get
            {
                return !String.IsNullOrEmpty(EncryptedLogin) && !String.IsNullOrEmpty(EncryptedPassword);
            }
        }
        public void SetPassword(string password)
        {
            EncryptedPassword = CredentialTool.EncryptStringForConfig(password);
        }

        public void SetLogin(string login)
        {
            EncryptedLogin = CredentialTool.EncryptStringForConfig(login);
        }

        public string GetPassword()
        {
            if (!HasCustomCredential)
            {
                return String.Empty;
            }
            return CredentialTool.DecryptStringForConfig(EncryptedPassword);
        }

        public string GetLogin()
        {
            if (!HasCustomCredential)
            {
                return String.Empty;
            }
            return CredentialTool.DecryptStringForConfig(EncryptedLogin);

        }

        #region IEquatable<ServerInfo> Members

        public bool Equals(ServerInfo other)
        {
            if (other == null)
            {
                return false;
            }

            return TFComparer.CompareTFSUrl(this.ServerAddress, other.ServerAddress) == 0;
        }

        #endregion
    }
}
