using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Fissum.ToolForms
{
    public partial class ServerDetailDialog : Form
    {
        public ServerDetailDialog()
        {
            InitializeComponent();
        }

        public string ServerName
        {
            get
            {
                if (string.IsNullOrEmpty(nameTextBox.Text))
                {
                    return ServerURL;
                }
                return nameTextBox.Text;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Name");
                }

                nameTextBox.Text = value;
            }
        }

        public string ServerURL
        {
            get
            {
                return urlTextBox.Text;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("URL");
                }

                urlTextBox.Text = value;
            }
        }


        private void TextBox1_Validating(object sender, CancelEventArgs e)
        {
            if (nameTextBox.Text.Length == 0)
            {
                errorProvider.SetError(nameTextBox, "Please fill the name");
                e.Cancel = true;
                return;
            }

            errorProvider.SetError(nameTextBox, String.Empty);
            e.Cancel = false;            
        }

        private void UrlTextBox_Validating(object sender, CancelEventArgs e)
        {
            if (urlTextBox.Text.Length == 0)
            {
                errorProvider.SetError(urlTextBox, "Please fill the url");
                e.Cancel = true;
                return;
            }

            if (!Uri.IsWellFormedUriString(urlTextBox.Text,UriKind.Absolute))
            {
                errorProvider.SetError(urlTextBox, "The specified value is not a valid URL");
                e.Cancel = true;
                return;
            }

            if ((!urlTextBox.Text.StartsWith("http://", StringComparison.OrdinalIgnoreCase)) &&
              (!urlTextBox.Text.StartsWith("https://", StringComparison.OrdinalIgnoreCase)))
            {
                errorProvider.SetError(nameTextBox, "The specified value is not a valid URL");
                e.Cancel = true;
                return;
            }

            errorProvider.SetError(urlTextBox, String.Empty);
            e.Cancel = false;
        }
    }
}