using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using System.Text;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Fissum.TeamSystem
{
    /// <summary>
    /// Represents a Workitem type.
    /// </summary>
    public sealed class WorkItemType
    {
        TFSTracking.WorkItemType _type;
        Project _project;


        /// <summary>
        /// Initializes a new instance of the <see cref="T:WorkItemType"/> class.
        /// </summary>
        /// <param name="type">The type.</param>
        internal WorkItemType(TFSTracking.WorkItemType type, Project project)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            _type = type;
            _project = project;
        }

        /// <summary>
        /// News the work item.
        /// </summary>
        /// <returns></returns>
        public WorkItem NewWorkItem()
        {
            TFSTracking.WorkItem wi;

            wi = _type.NewWorkItem();
            return WorkItem.Create(wi,_project,true);
        }

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return _type.Name;
            }
        }

        /// <summary>
        /// Gets the description.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get
            {
                return _type.Description;
            }
        }

        /// <summary>
        /// Gets the field of the type.
        /// </summary>
        /// <value>The fields.</value>
        public IEnumerable<WorkItemFieldDefinition> Fields
        {
            get
            {

                foreach (TFSTracking.FieldDefinition fieldDef in _type.FieldDefinitions)
                {
                    yield return new WorkItemFieldDefinition(fieldDef);
                }
            }
        }

    }

}