using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using Fissum.TeamSystem;

namespace Fissum.Core
{
    public class WorkItemComparer : IComparer, IComparer<WorkItem>
    {
        private WorkItemFieldDefinition _field;

        private bool _compareAscending;

        public bool CompareAscending
        {
            get
            {
                return _compareAscending;
            }
            set
            {
                if (_compareAscending == value)
                {
                    return;
                }
                _compareAscending = value;
            }
        }

        public WorkItemFieldDefinition Field
        {
            get
            {
                return _field;
            }
            set
            {
                _field = value;
            }
        }

        #region IComparer Members

        public int Compare(object x, object y)
        {         
            WorkItem wiX = x as WorkItem;
            WorkItem wiY = y as WorkItem;

            return Compare(wiX, wiY);
        }

        #endregion

        #region IComparer<WorkItemObject> Members

        public int Compare(WorkItem first, WorkItem second)
        {
            int result;
            if (first == null && second != null)
            {
                result =  - 1;
            }
            else if (first != null && second == null)
            {
                result = 1;
            }
            else if (first == null && second == null)
            {
                return 0;
            }
            else
            {
                if (_field == null)
                {
                   return 0;
                }

                IComparable firstFieldToCompare = first[_field.Name] as IComparable;
                IComparable secondFieldToCompare = second[_field.Name] as IComparable;

             
                if (firstFieldToCompare == secondFieldToCompare)
                {
                    result = 0;
                }
                else if (firstFieldToCompare == null)
                {
                    result = -1;
                }
                else if (secondFieldToCompare == null)
                {
                    result = 1;
                }
                else
                {
                    result = firstFieldToCompare.CompareTo(secondFieldToCompare);
                }
            }
            if (!_compareAscending)
            {
                return -result;
            }
            return result;
        }

        #endregion
    }
}
