﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.Lib;
using System.IO;
using System.Data;

namespace Fissum
{
    public static class FissumApplication
    {
        private static Fissum.Core.Context _context;
        private static ConfigurationDataSet _applicationConfiguration;

        public static void SetContext(Fissum.Core.Context context)
        {
            if (_context != null)
            {
                throw new InvalidOperationException();
            }

            _context = context;
        }


        public static Fissum.Core.Context Context
        {
            get
            {
                return _context;
            }       
        }

        public static ConfigurationDataSet Configuration
        {
            get
            {
                return _applicationConfiguration;
            }
            set
            {
                _applicationConfiguration = value;
            }
        }

        /// <summary>
        /// Returns the path to the fissum configuration path
        /// </summary>
        public static string DataDirectory
        {
            get
            {
                string appDataPath = System.Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
                return Path.Combine(appDataPath, "FissumApplication.2005");
            }
        }

        /// <summary>
        /// return the name of the fissum configuration file
        /// </summary>
        public static string ConfigurationFileName
        {
            get
            {
                return Path.Combine(DataDirectory, "core.config");
            }
        }

        /// <summary>
        /// Checks if the fissum data directory exists, the directory is created if not
        /// </summary>
        public static void CheckFissumApplicationDirectory()
        {
            DirectoryInfo dirInfo = new DirectoryInfo(DataDirectory);
            if (!dirInfo.Exists)
            {
                dirInfo.Create();              
            }
        }

        public static bool SaveConfiguration()
        {
            try
            {
                CheckFissumApplicationDirectory();
                _applicationConfiguration.WriteXml(ConfigurationFileName, XmlWriteMode.IgnoreSchema);
                
                return true;
            }
            catch
            {
                
                return false;
            }
        }


        public static bool LoadConfiguration()
        {
            try
            {
                string configurationFile = ConfigurationFileName;
                _applicationConfiguration = new ConfigurationDataSet();

                if (!File.Exists(configurationFile))
                {                   
                    SaveConfiguration();
                }
                else
                {
                    _applicationConfiguration.ReadXml(configurationFile, XmlReadMode.IgnoreSchema);
                    return true;
                }
            }
            catch
            {
                _applicationConfiguration = new ConfigurationDataSet();
                SaveConfiguration();
            }
            return false;
        }
    }
}
