using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;
using Fissum.Core;
using Fissum.TeamSystem;

namespace Fissum.Lib
{
    /// <summary>
    /// Class used to sort workitems in a listview
    /// </summary>
    internal class WorkItemListViewItemComparer : IComparer
    {
        private WorkItemComparer _comparer;

        #region IComparer Members

        public WorkItemListViewItemComparer(WorkItemComparer comparer)
        {
            if (comparer == null)
            {
                throw new ArgumentNullException("comparer");
            }

            _comparer = comparer;
        }

        /// <summary>
        /// Compare method
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public int Compare(object x, object y)
        {
            ListViewItem itemX = x as ListViewItem;
            ListViewItem itemY = y as ListViewItem;

            if (itemX == null)
            {
                return 1;
            }

            if (itemY == null)
            {
                return -1;
            }

            WorkItem wiX = itemX.Tag as WorkItem;
            WorkItem wiY = itemY.Tag as WorkItem;

            if (wiX == null)
            {
                return 1;
            }

            if (wiY == null)
            {
                return -1;
            }


            return _comparer.Compare(wiX, wiY);
        }

        #endregion
    }
}
