using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Fissum.Lib;
using Fissum.TeamSystem;

namespace Fissum
{
    class ProjectWssSiteToolStripMenuItem: ToolStripMenuItem
    {



        private Project _project;




        public ProjectWssSiteToolStripMenuItem(Project project)
            : base()
        {

            _project = project;


            
            RefreshSite();
        }



        public void RefreshSite()
        {
            DropDownItems.Clear();
            Text = "Loading...";
            Enabled = false;
            Program.MainForm.AsyncWorkPool.Push(new LoadWssSiteTask(this));
        }


        private class LoadWssSiteTask : IAsyncWorkTask
        {
            ProjectWssSiteToolStripMenuItem _toolStrip;


            public LoadWssSiteTask(ProjectWssSiteToolStripMenuItem toolStrip)
            {
                _toolStrip = toolStrip;
            }


            #region IAsyncWorkTask Members

            public void Run(AsyncWorkPool pool)
            {
                _toolStrip._project.LoadProjectDocuments();          
            }

            public void Completed(AsyncWorkPool pool)
            {
                if (!_toolStrip._project.WssSiteOnline)
                {                    
                    _toolStrip.Text = "No Team Site Found";
                    return;
                }

                _toolStrip.Visible = true;
                _toolStrip.Text = "Team Site";
                _toolStrip.Enabled = true;

                ToolStripMenuItem siteMenuItem = new ToolStripMenuItem("Open Team Site");

                siteMenuItem.Tag = _toolStrip._project;
                siteMenuItem.ToolTipText = _toolStrip._project.ProjectSiteUrl;                

                siteMenuItem.Click += new EventHandler(SiteMenuItem_Click);
                siteMenuItem.Image = GlobalRessources.Web;

                _toolStrip.DropDownItems.Add(siteMenuItem);               

                if (_toolStrip._project.DocumentList.Count > 0)
                {
                    _toolStrip.DropDownItems.Add(new ToolStripSeparator());
                }

                foreach (DocumentList docList in _toolStrip._project.DocumentList)
                {
                    ToolStripMenuItem docListMenuItem = new ToolStripMenuItem(docList.Title);
                    docListMenuItem.Image = GlobalRessources.doclist;                    
                    FillDocFolder(docList.RootFolder, docListMenuItem);
                    _toolStrip.DropDownItems.Add(docListMenuItem);
                }

                if (_toolStrip._project.DocumentList.Count > 0)
                {
                    _toolStrip.DropDownItems.Add(new ToolStripSeparator());

                    ToolStripMenuItem refreshSite = new ToolStripMenuItem("Refresh");
                    refreshSite.Tag = _toolStrip;
                    refreshSite.Click += new EventHandler(RefreshSite_Click);
                    refreshSite.Image = GlobalRessources.SychronizeListHS;
                    _toolStrip.DropDownItems.Add(refreshSite);
                }
            }

            void RefreshSite_Click(object sender, EventArgs e)
            {
                ToolStripMenuItem toolStripItem = sender as ToolStripMenuItem;
                if (sender == null)
                {
                    return;
                }

                ProjectWssSiteToolStripMenuItem wssMenuItem = toolStripItem.Tag as ProjectWssSiteToolStripMenuItem;
                wssMenuItem.RefreshSite();
            }

            private void FillDocFolder(DocumentFolder docFolder, ToolStripMenuItem docFolderMenuItem)
            {
                if (docFolder.IsEmpty)
                {
                    ToolStripMenuItem nofileMenuItem = new ToolStripMenuItem("Empty");
                    nofileMenuItem.Enabled = false;
                    docFolderMenuItem.DropDownItems.Add(nofileMenuItem);
                }

                foreach (DocumentFolder childFolder in docFolder.FolderList)
                {
                    ToolStripMenuItem childFolderMenuItem = new ToolStripMenuItem(childFolder.Name);
                    childFolderMenuItem.Image = GlobalRessources.folder;
                    FillDocFolder(childFolder, childFolderMenuItem);
                    docFolderMenuItem.DropDownItems.Add(childFolderMenuItem);
                }

                foreach (Document doc in docFolder.DocumentList)
                {
                    ToolStripMenuItem documentMenuItem = new ToolStripMenuItem(doc.Name);
                    int pos = doc.Name.LastIndexOf('.');
                    if (pos > 0)
                    {
                        string extension = doc.Name.Substring(pos);

                        Icon icon = IconHandler.GetIcon(extension, IconSize.Small);

                        if (icon != null)
                        {
                            documentMenuItem.Image = icon.ToBitmap();
                        }

                    }
                    documentMenuItem.Tag = doc;
                    documentMenuItem.Click += new EventHandler(DocumentMenuItem_Click);
                    docFolderMenuItem.DropDownItems.Add(documentMenuItem);
                }

            }

            void DocumentMenuItem_Click(object sender, EventArgs e)
            {
                ToolStripMenuItem menuItem = sender as ToolStripMenuItem;
                if (menuItem == null)
                {
                    return;
                }

                Document document = menuItem.Tag as Document;
                if (document == null)
                {
                    return;
                }

                document.Open();

               


            }

            void SiteMenuItem_Click(object sender, EventArgs e)
            {
                ProcessStartInfo startInfo = new ProcessStartInfo(_toolStrip._project.ProjectSiteUrl);
                startInfo.Verb = "Open";
                Process.Start(startInfo);
            }

            #endregion
        }
    }
}
