﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Fissum.TeamSystem;
using Fissum.Model.Generator;
using System.Windows.Markup;
using System.IO;
using System.Text.RegularExpressions;

namespace Fissum.Model.BuilderUI
{
    public partial class MainForm : Form
    {
        private Fissum.Core.Context _context;

        public MainForm()
        {
            InitializeComponent();
            Enabled = false;
        }

        /// <summary>
        /// Handles the Load event of the MainForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            _context = new Fissum.Core.Context(Fissum.Core.ContextMode.Normal);

            ConnectingForm connectingForm = new ConnectingForm(_context);

            connectingForm.ModelToOpenName = ModelPathCmdArgument;

            using (connectingForm)
            {
                if (connectingForm.ShowDialog() != DialogResult.OK)
                {
                    Application.Exit();
                    return;
                }

                bsModel.DataSource = connectingForm.ModelInfo;
                Enabled = true;
            }

        }
        public string ModelPathCmdArgument { get; set; }

        private ModelInfo CurrentModel
        {
            get
            {
                return bsModel.DataSource as ModelInfo;
            }
        }


        private void BtnGenerate_Click(object sender, EventArgs e)
        {
            if (CurrentModel == null)
            {
                return;
            }

            string modelNameSpace = tbNamespace.Text.Trim();
            if (!ValidateNamespace(modelNameSpace))
            {
                MessageBox.Show("Invalid namespace");
                return;
            }

            if (String.IsNullOrEmpty(CurrentModel.FileName))
            {
                sfdModel.FileName = CurrentModel.Project;
                if (sfdModel.ShowDialog() != DialogResult.OK)
                {
                    return;
                }
                CurrentModel.FileName = sfdModel.FileName;
                bsModel.ResetBindings(false);
            }

            foreach (var info in CurrentModel.WIList)
            {
                if (!info.IsValid)
                {
                    MessageBox.Show(String.Format("Informations for '{0}' are not valid", info.WITypeName));
                    return;
                }
            }


            ModelDefinition modelDefinition = CurrentModel.GenerateModelDefinition();
            
            string xaml = XamlWriter.Save(modelDefinition);

            File.WriteAllText(CurrentModel.FileName, xaml);

        }

        private bool ValidateNamespace(string modelNameSpace)
        {
            return !String.IsNullOrEmpty(CurrentModel.Namespace);
        }

        private void DgvWorkItem_DoubleClick(object sender, EventArgs e)
        {
            WorkItemInfo wiInfo = bsWorkItemType.Current as WorkItemInfo;
            if (wiInfo == null)
            {
                return;
            }

            using (var form = new PropertySelectionForm())
            {
                form.CurrentWIInfo = wiInfo;
                form.ShowDialog();
            }
        } 
    }
}