using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Linq;
using Fissum.TeamSystem;
using Fissum.Lib;

namespace Fissum
{
    internal class QueryNodeMenuItem : ToolStripMenuItem
    {
        private List<QueryNodeMenuItem> _nodeList;
        private List<WorkItemQueryToolStripMenuItem> _queryList;

        public QueryNodeMenuItem(string name)
        {
            Text = name;
            _nodeList = new List<QueryNodeMenuItem>();
        }

        /// <summary>
        /// Attach a query to the node
        /// </summary>
        /// <param name="query"></param>
        /// <param name="nameList"></param>
        public void AttachQuery(WorkItemQuery query, List<string> nameList)
        {
            if (nameList.Count == 1)
            {
                if (_queryList == null)
                {
                    _queryList = new List<WorkItemQueryToolStripMenuItem>();
                }

                _queryList.Add(new WorkItemQueryToolStripMenuItem(query, nameList[0]));

            }
            else
            {
                QueryNodeMenuItem node = (from n in _nodeList
                                          where n.Text == nameList[0]
                                          select n).FirstOrDefault();

                if (node == null)
                {
                    node = new QueryNodeMenuItem(nameList[0]);
                    _nodeList.Add(node);
                }

                node.AttachQuery(query, nameList.Skip(1).ToList());
            }

        }

        /// <summary>
        /// Build the toolstripmenu with the attached nodes and queries
        /// </summary>
        /// <param name="rootNode"></param>
        public void BuildMenu()
        {
            BuildMenu(null);
        }

        /// <summary>
        /// Build the toolstripmenu with the attached nodes and queries
        /// </summary>
        /// <param name="rootNode"></param>
        public void BuildMenu(ToolStripMenuItem rootNode)
        {
            if (rootNode == null)
            {
                rootNode = this;
            }
            var finalNodeList = (from node in _nodeList
                                 orderby node.Text
                                 select node);

            foreach (var node in finalNodeList)
            {
                node.BuildMenu();
                rootNode.DropDownItems.Add(node);
            }
            if (_queryList != null)
            {
                foreach (var query in _queryList)
                {
                    rootNode.DropDownItems.Add(query);
                }
            }
        }
    }
}
