using System;
using System.Collections.Generic;
using System.Text;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Net;
using System.Diagnostics;
using System.Collections;
using System.Runtime.Serialization;
using System.Linq;

namespace Fissum.TeamSystem
{
    [DebuggerDisplay(@"\{{Name}: {ReferenceName}\}")]
    public class WorkItemFieldDefinition: IEquatable<WorkItemFieldDefinition>
    {
        TFSTracking.FieldDefinition _field;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:WorkItemQueryField"/> class.
        /// </summary>
        /// <param name="field">The field.</param>
        internal WorkItemFieldDefinition(TFSTracking.FieldDefinition field)
        {
            _field = field;            
        }

        /// <summary>
        /// Return a field from its reference type
        /// </summary>
        /// <param name="referenceName"></param>
        /// <returns></returns>
        public static WorkItemFieldDefinition GetByReferenceName(WorkItemObject wi, string referenceName)
        {
            TFSTracking.FieldDefinition def = (from TFSTracking.Field fieldItem in wi.TFSWorkItem.Fields
                                               where fieldItem.ReferenceName == referenceName
                                               select fieldItem.FieldDefinition).FirstOrDefault();

            if (def != null)
            {
                return new WorkItemFieldDefinition(def);
            }

            return null;
        }

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return _field.Name;
            }
        }

        public bool CanSortBy
        {
            get
            {
                return _field.CanSortBy;
            }
        }

        public Type FieldType
        {
            get
            {
                return _field.SystemType;
            }
        }


        public const string ReferenceNameArea = "System.AreaId";
        public const string ReferenceNameIteration = "System.IterationId";
        public string ReferenceName
        {
            get
            {
                return _field.ReferenceName;
            }
        }

        public bool IsEditable
        {
            get
            {
                return _field.IsEditable;
            }
        }

        public bool IsComputed
        {
            get
            {
                return _field.IsComputed;
            }
        }


        public bool IsQueryable
        {
            get
            {
                return _field.IsQueryable;
            }
        }
        public List<T> GetAllowedValues<T>()
        {
            if (typeof(T) != _field.SystemType)
            {
                return null;
            }

            List<T> result = new List<T>();
            TFSTracking.AllowedValuesCollection coll = _field.AllowedValues;
            foreach (object obj in coll)
            {
                try
                {
                    T resultValue = (T)obj;
                    result.Add(resultValue);
                }
                catch
                {

                }
            }
            return result;
        }

        public string HelpText
        {
            get
            {
                return _field.HelpText;
            }
        }

        internal TFSTracking.FieldDefinition TFSField
        {
            get
            {
                return _field;
            }
        }

        #region IEquatable<WorkItemFieldDefinition> Members

        public bool Equals(WorkItemFieldDefinition other)
        {
            if (other == null)
            {
                return false;
            }

            return other.ReferenceName == this.ReferenceName;
        }

        #endregion
    }
}
