﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.TeamSystem.Collections;
using Fissum.Core.Data;
using Fissum.TeamSystem;

namespace Fissum.Lib
{
    internal class History
    {
        private static WorkItemCollection _wiHistoryCollection;
        private static List<WorkItemFieldDefinition> _fieldDefinitionList;

        static History()
        {
            _wiHistoryCollection = new WorkItemCollection();
            _fieldDefinitionList = new List<WorkItemFieldDefinition>();
        }



        public static event EventHandler HistoryChanged;

        #region OnHistoryChanged
        /// <summary>
        /// Triggers the HistoryChanged event.
        /// </summary>
        public static void OnHistoryChanged()
        {
            if (HistoryChanged != null)
            {
                HistoryChanged(null, EventArgs.Empty);
            }
        }
        #endregion

        public class WorkItemHistoryCollectionSource : WorkItemCollectionSource
        {

            protected override WorkItemCollectionResult RefreshSource()
            {
                WorkItemCollectionResult result = new WorkItemCollectionResult();
                result.Collection = _wiHistoryCollection.Clone();
                result.Collection.Reverse();
                result.FieldList = _fieldDefinitionList;
                result.SortFieldList = new List<WorkItemSortField>();
                /// result.FieldList.Add(new Fissum.TeamSystem.WorkItemFieldDefinition())
                /// 

                return result;
            }
        }

        public static void Add(WorkItemObject wi)
        {
            if (wi.IsNew)
            {
                return;
            }

            foreach (WorkItemObject hwi in _wiHistoryCollection)
            {
                if ((hwi.Project == wi.Project) && (wi.Id == hwi.Id))
                {
                    return;
                }
            }

            if (_wiHistoryCollection.Count == 0)
            {
                WorkItemFieldDefinition field;

                string[] referenceNames = { "System.Id", "System.WorkItemType", "System.AssignedTo","System.Title" };

                foreach (string refName in referenceNames)
                {
                    field = WorkItemFieldDefinition.GetByReferenceName(wi, refName);
                    if (field != null)
                    {
                        _fieldDefinitionList.Add(field);

                    }
                }
            }

            _wiHistoryCollection.Add(wi);
            OnHistoryChanged();
        }

        public static void Clear()
        {
            _wiHistoryCollection.Clear();
            OnHistoryChanged();
        }

        public static int Count
        {
            get
            {
                return _wiHistoryCollection.Count;
            }
        }

        public static WorkItemCollectionSource HistorySource
        {
            get
            {
                return new WorkItemHistoryCollectionSource();
            }
        }
    }
}
