using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Fissum.TeamSystem;
using System.Diagnostics;

namespace Fissum
{
    static class Program
    {
        static private MainForm _mainForm;
        static private bool _discoveryMode;

        /// <summary>
        /// Gets the main form.
        /// </summary>
        /// <value>The main form.</value>
        static public MainForm MainForm
        {
            [DebuggerStepThrough()]
            get
            {
                return _mainForm;
            }
        }

        static public bool DiscoveryMode
        {
            get
            {
                return _discoveryMode;
            }
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            if (args.Length != 1)
            {
                _discoveryMode = false;
            }
            else
            {
                _discoveryMode = args[0].ToString() == "/discovery";
            }

            FissumApplication.SetContext(new Fissum.Core.Context(_discoveryMode ? Fissum.Core.ContextMode.Discovery : Fissum.Core.ContextMode.Normal));
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);


            if (!FissumApplication.Context.IsTeamExplorerInstalled)
            {
                MessageBox.Show("Team Explorer is not installed!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _mainForm = new MainForm();
            Application.Run(_mainForm);

            Lib.UserPreferences prefs = Lib.UserPreferences.Default;
            if (prefs != null)
            {
                prefs.Save();
            }
        }
    }
}