﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Fissum.TeamSystem;

namespace Fissum.Core.Winform
{
    /// <summary>
    /// This user control can show a list of work items and show their details
    /// </summary>
    public partial class WorkItemMasterDetailCollectionListControl
        : UserControl
    {
        /// <summary>
        /// Default constructor
        /// </summary>
        public WorkItemMasterDetailCollectionListControl()
        {
            InitializeComponent();
        }

        [Browsable(true)]
        [Category(ComponantConstantData.PropertyCategory)]
        public WorkItemCollectionListControl WorkItemCollectionList
        {
            get
            {
                return workItemCollectionList;
            }
        }

        [Browsable(true)]
        [Category(ComponantConstantData.PropertyCategory)]
        public WorkItemObjectUserControl WorkItemObjectUserControl
        {
            get
            {
                return workItemObjectUserControl;
            }
        }



        private void WorkItemCollectionList_SelectedWorkItemChanged(object sender, EventArgs e)
        {
            List<WorkItemObject> selectedWorkItems = workItemCollectionList.SelectedWorkItems;
            if (selectedWorkItems.Count > 0)
            {
                workItemObjectUserControl.Enabled = true;
                workItemObjectUserControl.WorkItem = selectedWorkItems[0];

                if (WorkItemDetailViewed != null)
                {
                    WorkItemEventArgs args = new WorkItemEventArgs(workItemObjectUserControl.WorkItem);
                    WorkItemDetailViewed(this, args);
                }
            }
            else
            {
                if (WorkItemDetailViewed != null)
                {
                    WorkItemEventArgs args = new WorkItemEventArgs(null);
                    WorkItemDetailViewed(this, args);
                }
                workItemObjectUserControl.Enabled = false;
            }
        }

        /// <summary>
        /// Returns the work item shown in the detail part
        /// </summary>
        public WorkItemObject DetailedWorkItem
        {
            get
            {
                return workItemObjectUserControl.WorkItem;
            }
        }

        /// <summary>
        /// Event fired when a work item is selected in the detail view
        /// </summary>
        [Category("Workitem")]
        public event EventHandler<WorkItemEventArgs> WorkItemDetailViewed;

        /// <summary>
        /// Event fired when a work item in the collection list is saved
        /// </summary>
        [Category("Workitem")]
        public event EventHandler<WorkItemEventArgs> WorkItemSaved;

        /// <summary>
        /// Event fired when a work item in the collection list is modified
        /// </summary>
        [Category("Workitem")]
        public event EventHandler<WorkItemEventArgs> WorkItemValueChanged;


        private void WorkItemCollectionList_WorkItemSaved(object sender, WorkItemEventArgs e)
        {
            if (WorkItemSaved != null)
            {
                WorkItemSaved(this, e);
            }
        }

        private void WorkItemCollectionList_WorkItemValueChanged(object sender, WorkItemEventArgs e)
        {
            if (WorkItemValueChanged != null)
            {
                WorkItemValueChanged(this, e);
            }
        }
  
    }
}
