﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.TeamSystem.Collections;
using Fissum.TeamSystem;

namespace Fissum.Core.Data
{
    /// <summary>
    /// Generic source used to build a collection source from a workitem collection
    /// </summary>
    public class GenericWorkItemCollectionSource: WorkItemCollectionSource
    {
        private WorkItemCollectionResult _result;

        /// <summary>
        /// Builds the result object from the collection
        /// </summary>
        /// <param name="collection"></param>
        public GenericWorkItemCollectionSource(WorkItemCollection collection)
        {
            if (collection.Count == 0)
            {
                throw new ArgumentException("collection must not be empty");
            }
            _result = new WorkItemCollectionResult();
            _result.Collection = collection;
            _result.FieldList = new List<Fissum.TeamSystem.WorkItemFieldDefinition>();
            _result.SortFieldList = new List<WorkItemSortField>();
            WorkItemObject firstWi = collection[0];

            WorkItemFieldDefinition field;

            string[] referenceNames;
            
            referenceNames = new string [] { "System.Id", "System.WorkItemType", "System.ChangedDate","System.AssignedTo", "System.Title" };

            foreach (string refName in referenceNames)
            {
                field = WorkItemFieldDefinition.GetByReferenceName(firstWi, refName);
                if (field != null)
                {
                    _result.FieldList.Add(field);
                }
            }

            referenceNames = new string[] { "System.ChangedDate" };

            foreach (string refName in referenceNames)
            {
                field = WorkItemFieldDefinition.GetByReferenceName(firstWi, refName);
                if (field != null)
                {
                    _result.SortFieldList.Add(new WorkItemSortField(field, SoftFieldEnum.Descending));
                }
            }
        }

        protected override WorkItemCollectionResult RefreshSource()
        {
            return _result;
        }
    }
}
