using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Fissum
{
    class AsyncWorkPool: IDisposable
    {
        private object SyncRoot = new object();
        private Queue<IAsyncWorkTask> _taskQueue;
        private bool _disposed;


        public AsyncWorkPool()
        {
            _disposed = false;
            _taskQueue = new Queue<IAsyncWorkTask>();

            Thread thread = new Thread(Run);
            thread.Start();
        }


        public void Push(IAsyncWorkTask task)
        {
            Monitor.Enter(SyncRoot);

            _taskQueue.Enqueue(task);
            Monitor.Pulse(SyncRoot);

            Monitor.Exit(SyncRoot);
        }


        private void Run()
        {
            while (!_disposed)
            {
                Monitor.Enter(SyncRoot);

                if (_taskQueue.Count == 0)
                {
                    Monitor.Wait(SyncRoot);
                }

                if (_taskQueue.Count == 0)
                {
                    Monitor.Exit(SyncRoot);
                    return;
                }

                IAsyncWorkTask task = _taskQueue.Dequeue();


                Monitor.Exit(SyncRoot);


                try
                {
                    task.Run(this);
                }
                catch
                {

                }

                try
                {
                    if (!Program.MainForm.IsDisposed)
                    {
                        Program.MainForm.Invoke(new MethodInvoker(delegate() { task.Completed(this); }));
                    }
                }
                catch
                {

                }
            }

        }


        public void Dispose()
        {
            _disposed = true;
            Monitor.Enter(SyncRoot);
            Monitor.Pulse(SyncRoot);
            Monitor.Exit(SyncRoot);
        }
    }


    interface IAsyncWorkTask
    {
        void Run(AsyncWorkPool pool);

        void Completed(AsyncWorkPool pool);
    }




}
