using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;

namespace Fissum.Lib
{
    /// <summary>
    /// Originaly from "http://www.codeproject.com/csharp/iconhandler.asp"
    /// </summary>
    internal struct SHFILEINFO
    {
        public IntPtr HIcon;
        public IntPtr IIcon;
        public uint DWAttributes;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
        public string SZDisplayName;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string SZTypeName;
    }

    public enum IconSize : int
    {
        Large = 0x0,  //32x32
        Small = 0x1 //16x16        
    }

    internal static class IconHandler
    {
        const uint SHGFI_ICON = 0x100;
        const uint SHGFI_USEFILEATTRIBUTES = 0x10;

        [DllImport("Shell32", CharSet = CharSet.Auto)]
        private extern static int ExtractIconEx(
            [MarshalAs(UnmanagedType.LPTStr)]
            string lpszFile,       //size of the icon
            int nIconIndex,        //index of the icon            
            IntPtr[] phIconLarge,  //32x32 icon
            IntPtr[] phIconSmall,  //16x16 icon
            int nIcons);           //how many to get

        [DllImport("shell32.dll")]
        static extern IntPtr SHGetFileInfo(
            string pszPath,                //path
            uint dwFileAttributes,        //attributes
            ref SHFILEINFO psfi,        //struct pointer
            uint cbSizeFileInfo,        //size
            uint uFlags);    //flags

        //we need this function to release the unmanaged resource,
        //the unmanaged resource will be 
        //copies to a managed one and it will be returned.
        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        extern static bool DestroyIcon(IntPtr handle);

        private static Icon IconFromExtension(string Extension, IconSize Size)
        {
            try
            {
                Icon TempIcon;

                //add '.' if nessesry


                //temp struct for getting file shell info
                SHFILEINFO TempFileInfo = new SHFILEINFO();

                SHGetFileInfo(
                    Extension,
                    0,
                    ref TempFileInfo,
                    (uint)Marshal.SizeOf(TempFileInfo),
                    SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | (uint)Size);

                TempIcon = (Icon)Icon.FromHandle(TempFileInfo.HIcon);
                return GetManagedIcon(ref TempIcon);
            }
            catch (Exception e)
            {
                System.Diagnostics.Debug.WriteLine("error" +
                       " while trying to get icon for " +
                       Extension + " :" + e.Message);
                return null;
            }
        }

        private static Icon GetManagedIcon(ref Icon UnmanagedIcon)
        {
            Icon ManagedIcon = (Icon)UnmanagedIcon.Clone();

            DestroyIcon(UnmanagedIcon.Handle);

            return ManagedIcon;
        }


        private static object _syncRoot;
        private static Dictionary<string, Icon> _smallIconDictionnary;
        private static Dictionary<string, Icon> _largeIconDictionnary;


        static IconHandler()
        {
            _smallIconDictionnary = new Dictionary<string, Icon>();
            _largeIconDictionnary = new Dictionary<string, Icon>();
            _syncRoot = new object();
        }

        static public Icon GetIcon(string extension, IconSize size)
        {
            if (extension == null)
            {
                throw new ArgumentNullException("extension");
            }

            if (extension.Length == 0)
            {
                throw new ArgumentException("extension");
            }

            if (extension[0] != '.')
            {
                extension = '.' + extension;
            }

            lock (_syncRoot)
            {
                Icon result = null;
                bool found = false;
                switch (size)
                {
                    case IconSize.Small:
                        {
                            found = _smallIconDictionnary.TryGetValue(extension, out result);

                            if (!found)
                            {
                                result = IconFromExtension(extension, size);
                                _smallIconDictionnary.Add(extension, result);
                            }
                        }
                        break;
                    case IconSize.Large:
                        {
                            found = _largeIconDictionnary.TryGetValue(extension, out result);

                            if (!found)
                            {
                                result = IconFromExtension(extension, size);
                                _largeIconDictionnary.Add(extension, result);
                            }
                        }
                        break;
                    default:
                        return null;
                }


                return result;
            }
        }
    }
}
