using System;
using System.Collections.Generic;
using System.Text;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Net;
using System.Diagnostics;
using System.Collections;

namespace Fissum.TeamSystem
{
    public class WorkItemEventArgs : EventArgs
    {
        private WorkItemObject _workItem;
        private WorkItemFieldDefinition _field;
        private Object _object;
        /// <summary>
        /// Initializes a new instance of the <see cref="T:WorkItemEventArgs"/> class.
        /// </summary>
        /// <param name="workItem">The work item.</param>
        /// 

        public WorkItemEventArgs(WorkItemObject workItem)
            : this(workItem, null, null)
        {

        }

        public WorkItemEventArgs(WorkItemObject workItem, WorkItemFieldDefinition field)
            : this(workItem, field, null)
        {

        }
        public WorkItemEventArgs(WorkItemObject workItem, WorkItemFieldDefinition field, Object obj)
            : base()
        {
            _workItem = workItem;
            _field = field;
            _object = obj;
        }

        public bool HasRelatedInfo
        {
            get
            {
                return _field != null || _object != null;
            }
        }

        /// <summary>
        /// Gets the work item.
        /// </summary>
        /// <value>The work item.</value>
        public WorkItemObject WorkItem
        {
            get
            {
                return _workItem;
            }
        }

        public WorkItemFieldDefinition Field
        {
            get
            {
                return _field;
            }
        }

        public object Object
        {
            get
            {
                return _object;
            }

        }


    }
}
